@@ -1,5 +1,5 @@
 """Errors - these are closely linked to what used to be called violations."""
-from typing import Optional, Tuple
+from typing import Optional, Tuple, Any, List
 
 CheckTuple = Tuple[str, int, int]
 
@@ -18,10 +18,12 @@ class SQLBaseError(ValueError):
         line_pos=0,
         ignore=False,
         fatal=False,
+        warning=False,
         **kwargs
     ):
         self.fatal = fatal
         self.ignore = ignore
+        self.warning = warning
         if pos:
             self.line_no, self.line_pos = pos.source_position()
         else:
@@ -81,24 +83,42 @@ class SQLBaseError(ValueError):
             "line_pos": self.line_pos,
             "code": self.rule_code(),
             "description": self.desc(),
+            "name": getattr(self, "rule").name if hasattr(self, "rule") else "",
         }
 
-    def ignore_if_in(self, ignore_iterable):
+    def check_tuple(self) -> CheckTuple:
+        """Get a tuple representing this error. Mostly for testing."""
+        return (
+            self.rule_code(),
+            self.line_no,
+            self.line_pos,
+        )
+
+    def source_signature(self) -> Tuple[Any, ...]:
+        """Return hashable source signature for deduplication."""
+        return (self.check_tuple(), self.desc())
+
+    def ignore_if_in(self, ignore_iterable: List[str]):
         """Ignore this violation if it matches the iterable."""
-        # Type conversion
-        if isinstance(ignore_iterable, str):  # pragma: no cover TODO?
-            ignore_iterable = []
-        # Ignoring
         if self._identifier in ignore_iterable:
             self.ignore = True
 
+    def warning_if_in(self, warning_iterable: List[str]):
+        """Warning only for this violation if it matches the iterable.
+
+        Designed for rule codes so works with L001, LL0X but also TMP or PRS
+        for templating and parsing errors.
+        """
+        if self.rule_code() in warning_iterable:
+            self.warning = True
+
 
 class SQLTemplaterError(SQLBaseError):
     """An error which occurred during templating.
 
     Args:
         pos (:obj:`PosMarker`, optional): The position which the error
-            occured at.
+            occurred at.
 
     """
 
@@ -117,7 +137,7 @@ class SQLLexError(SQLBaseError):
 
     Args:
         pos (:obj:`PosMarker`, optional): The position which the error
-            occured at.
+            occurred at.
 
     """
 
@@ -182,13 +202,21 @@ class SQLLintError(SQLBaseError):
             return True
         return False
 
-    def check_tuple(self) -> CheckTuple:
-        """Get a tuple representing this error. Mostly for testing."""
-        return (
-            self.rule.code,
-            self.line_no,
-            self.line_pos,
+    def source_signature(self) -> Tuple[Any, ...]:
+        """Return hashable source signature for deduplication.
+
+        For linting errors we need to dedupe on more than just location and
+        description, we also need to check the edits potentially made, both
+        in the templated file but also in the source.
+        """
+        fix_raws = tuple(
+            tuple(e.raw for e in f.edit) if f.edit else None for f in self.fixes
         )
+        source_fixes = tuple(
+            tuple(tuple(e.source_fixes) for e in f.edit) if f.edit else None
+            for f in self.fixes
+        )
+        return (self.check_tuple(), self.description, fix_raws, source_fixes)
 
     def __repr__(self):
         return "<SQLLintError: rule {} pos:{!r}, #fixes: {}, description: {}>".format(
