@@ -14,6 +14,7 @@ from typing import (
     Set,
     Tuple,
     Type,
+    Dict,
     cast,
 )
 
@@ -33,14 +34,13 @@ from sqlfluff.core.parser import Lexer, Parser, RegexLexer
 from sqlfluff.core.file_helpers import get_encoding
 from sqlfluff.core.templaters import TemplatedFile
 from sqlfluff.core.rules import get_ruleset
-from sqlfluff.core.rules.doc_decorators import is_fix_compatible
 from sqlfluff.core.config import FluffConfig, ConfigLoader, progress_bar_configuration
 
 # Classes needed only for type checking
 from sqlfluff.core.parser.segments.base import BaseSegment, SourceFix
 from sqlfluff.core.parser.segments.meta import MetaSegment
 from sqlfluff.core.parser.segments.raw import RawSegment
-from sqlfluff.core.rules import BaseRule
+from sqlfluff.core.rules import BaseRule, RulePack
 
 from sqlfluff.core.linter.common import (
     RuleTuple,
@@ -48,12 +48,17 @@ from sqlfluff.core.linter.common import (
     NoQaDirective,
     RenderedFile,
 )
-from sqlfluff.core.linter.linted_file import LintedFile
+from sqlfluff.core.linter.linted_file import (
+    LintedFile,
+    FileTimings,
+    TMP_PRS_ERROR_TYPES,
+)
 from sqlfluff.core.linter.linted_dir import LintedDir
 from sqlfluff.core.linter.linting_result import LintingResult
 
 
 WalkableType = Iterable[Tuple[str, Optional[List[str]], List[str]]]
+RuleTimingsType = List[Tuple[str, str, float]]
 
 # Instantiate the linter logger
 linter_logger: logging.Logger = logging.getLogger("sqlfluff.linter")
@@ -93,19 +98,22 @@ class Linter:
         # Store references to user rule classes
         self.user_rules = user_rules or []
 
-    def get_ruleset(self, config: Optional[FluffConfig] = None) -> List[BaseRule]:
+    def get_rulepack(self, config: Optional[FluffConfig] = None) -> RulePack:
         """Get hold of a set of rules."""
         rs = get_ruleset()
         # Register any user rules
         for rule in self.user_rules:
             rs.register(rule)
         cfg = config or self.config
-        return rs.get_rulelist(config=cfg)
+        return rs.get_rulepack(config=cfg)
 
     def rule_tuples(self) -> List[RuleTuple]:
         """A simple pass through to access the rule tuples of the rule set."""
-        rs = self.get_ruleset()
-        return [RuleTuple(rule.code, rule.description) for rule in rs]
+        rs = self.get_rulepack()
+        return [
+            RuleTuple(rule.code, rule.name, rule.description, rule.groups, rule.aliases)
+            for rule in rs.rules
+        ]
 
     # #### Static methods
     # These are the building blocks of the linting process.
@@ -163,7 +171,7 @@ class Linter:
             linter_logger.info(
                 "Lexed tokens: %s", [seg.raw for seg in tokens] if tokens else None
             )
-        except SQLLexError as err:
+        except SQLLexError as err:  # pragma: no cover
             linter_logger.info("LEXING FAILED! (%s): %s", templated_file.fname, err)
             violations.append(err)
             return None, violations, config
@@ -185,7 +193,7 @@ class Linter:
                 getattr(elem, "indent_val", 0)
                 for elem in cast(Tuple[BaseSegment, ...], tokens)
             )
-            if indent_balance != 0:
+            if indent_balance != 0:  # pragma: no cover
                 linter_logger.debug(
                     "Indent balance test failed for %r. Template indents will not be "
                     "linted for this file.",
@@ -203,7 +211,7 @@ class Linter:
                 if token.indent_val != 0:
                     # Don't allow it if we're not linting templating block indents.
                     if not templating_blocks_indent:
-                        continue
+                        continue  # pragma: no cover
             new_tokens.append(token)
 
         # Return new buffer
@@ -258,13 +266,13 @@ class Linter:
     def parse_noqa(
         comment: str,
         line_no: int,
-        rule_codes: List[str],
+        reference_map: Dict[str, Set[str]],
     ):
         """Extract ignore mask entries from a comment string."""
         # Also trim any whitespace afterward
 
         # Comment lines can also have noqa e.g.
-        # --dafhsdkfwdiruweksdkjdaffldfsdlfjksd -- noqa: L016
+        # --dafhsdkfwdiruweksdkjdaffldfsdlfjksd -- noqa: LT05
         # Therefore extract last possible inline ignore.
         comment = [c.strip() for c in comment.split("--")][-1]
 
@@ -306,22 +314,25 @@ class Linter:
                         unexpanded_rules = tuple(
                             r.strip() for r in rule_part.split(",")
                         )
-                        expanded_rules = []
+                        # We use a set to do natural deduplication.
+                        expanded_rules: Set[str] = set()
                         for r in unexpanded_rules:
-                            expanded_rule = [
-                                x
-                                for x in fnmatch.filter(rule_codes, r)
-                                if x not in expanded_rules
-                            ]
-                            if expanded_rule:
-                                expanded_rules.extend(expanded_rule)
-                            elif r not in expanded_rules:
+                            matched = False
+                            for expanded in (
+                                reference_map[x]
+                                for x in fnmatch.filter(reference_map.keys(), r)
+                            ):
+                                expanded_rules |= expanded
+                                matched = True
+
+                            if not matched:
                                 # We were unable to expand the glob.
                                 # Therefore assume the user is referencing
                                 # a special error type (e.g. PRS, LXR, or TMP)
                                 # and add this to the list of rules to ignore.
-                                expanded_rules.append(r)
-                        rules = tuple(expanded_rules)
+                                expanded_rules.add(r)
+                        # Sort for consistency
+                        rules = tuple(sorted(expanded_rules))
                     else:
                         rules = None
                     return NoQaDirective(line_no, rules, action)
@@ -375,7 +386,7 @@ class Linter:
         t0 = time.monotonic()
         violations = cast(List[SQLBaseError], rendered.templater_violations)
         tokens: Optional[Sequence[BaseSegment]]
-        if rendered.templated_file:
+        if rendered.templated_file is not None:
             tokens, lvs, config = cls._lex_templated_file(
                 rendered.templated_file, rendered.config
             )
@@ -416,13 +427,13 @@ class Linter:
     def extract_ignore_from_comment(
         cls,
         comment: RawSegment,
-        rule_codes: List[str],
+        reference_map: Dict[str, Set[str]],
     ):
         """Extract ignore mask entries from a comment segment."""
         # Also trim any whitespace afterward
         comment_content = comment.raw_trimmed().strip()
         comment_line, _ = comment.pos_marker.source_position()
-        result = cls.parse_noqa(comment_content, comment_line, rule_codes)
+        result = cls.parse_noqa(comment_content, comment_line, reference_map)
         if isinstance(result, SQLParseError):
             result.segment = comment
         return result
@@ -431,14 +442,14 @@ class Linter:
     def extract_ignore_mask_tree(
         cls,
         tree: BaseSegment,
-        rule_codes: List[str],
+        reference_map: Dict[str, Set[str]],
     ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
         """Look for inline ignore comments and return NoQaDirectives."""
         ignore_buff: List[NoQaDirective] = []
         violations: List[SQLBaseError] = []
         for comment in tree.recursive_crawl("comment"):
             if comment.is_type("inline_comment"):
-                ignore_entry = cls.extract_ignore_from_comment(comment, rule_codes)
+                ignore_entry = cls.extract_ignore_from_comment(comment, reference_map)
                 if isinstance(ignore_entry, SQLParseError):
                     violations.append(ignore_entry)
                 elif ignore_entry:
@@ -452,7 +463,7 @@ class Linter:
         cls,
         source: str,
         inline_comment_regex: RegexLexer,
-        rule_codes: List[str],
+        reference_map: Dict[str, Set[str]],
     ) -> Tuple[List[NoQaDirective], List[SQLBaseError]]:
         """Look for inline ignore comments and return NoQaDirectives.
 
@@ -465,7 +476,7 @@ class Linter:
             match = inline_comment_regex.search(line) if line else None
             if match:
                 ignore_entry = cls.parse_noqa(
-                    line[match[0] : match[1]], idx + 1, rule_codes
+                    line[match[0] : match[1]], idx + 1, reference_map
                 )
                 if isinstance(ignore_entry, SQLParseError):
                     violations.append(ignore_entry)  # pragma: no cover
@@ -480,12 +491,12 @@ class Linter:
         cls,
         tree: BaseSegment,
         config: FluffConfig,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         fname: Optional[str] = None,
         templated_file: Optional[TemplatedFile] = None,
         formatter: Any = None,
-    ) -> Tuple[BaseSegment, List[SQLBaseError], List[NoQaDirective]]:
+    ) -> Tuple[BaseSegment, List[SQLBaseError], List[NoQaDirective], RuleTimingsType]:
         """Lint and optionally fix a tree object."""
         # Keep track of the linting errors on the very first linter pass. The
         # list of issues output by "lint" and "fix" only includes issues present
@@ -496,6 +507,8 @@ class Linter:
         last_fixes = None
         # Keep a set of previous versions to catch infinite loops.
         previous_versions: Set[Tuple[str, Tuple[SourceFix, ...]]] = {(tree.raw, ())}
+        # Keep a buffer for recording rule timings.
+        rule_timings: RuleTimingsType = []
 
         # If we are fixing then we want to loop up to the runaway_limit, otherwise just
         # once for linting.
@@ -503,12 +516,13 @@ class Linter:
 
         # Dispatch the output for the lint header
         if formatter:
-            formatter.dispatch_lint_header(fname)
+            formatter.dispatch_lint_header(fname, sorted(rule_pack.codes()))
 
         # Look for comment segments which might indicate lines to ignore.
         if not config.get("disable_noqa"):
-            rule_codes = [r.code for r in rule_set]
-            ignore_buff, ivs = cls.extract_ignore_mask_tree(tree, rule_codes)
+            ignore_buff, ivs = cls.extract_ignore_mask_tree(
+                tree, rule_pack.reference_map
+            )
             initial_linting_errors += ivs
         else:
             ignore_buff = []
@@ -528,10 +542,10 @@ class Linter:
         for phase in phases:
             if len(phases) > 1:
                 rules_this_phase = [
-                    rule for rule in rule_set if rule.lint_phase == phase
+                    rule for rule in rule_pack.rules if rule.lint_phase == phase
                 ]
             else:
-                rules_this_phase = rule_set
+                rules_this_phase = rule_pack.rules
             for loop in range(loop_limit if phase == "main" else 2):
 
                 def is_first_linter_pass():
@@ -547,7 +561,7 @@ class Linter:
                 if is_first_linter_pass():
                     # In order to compute initial_linting_errors correctly, need
                     # to run all rules on the first loop of the main phase.
-                    rules_this_phase = rule_set
+                    rules_this_phase = rule_pack.rules
                 progress_bar_crawler = tqdm(
                     rules_this_phase,
                     desc="lint by rules",
@@ -564,11 +578,12 @@ class Linter:
                     if (
                         fix
                         and not is_first_linter_pass()
-                        and not is_fix_compatible(crawler)
+                        and not crawler.is_fix_compatible
                     ):
                         continue
 
                     progress_bar_crawler.set_description(f"rule {crawler.code}")
+                    t0 = time.monotonic()
 
                     # fixes should be a dict {} with keys edit, delete, create
                     # delete is just a list of segments to delete
@@ -598,8 +613,13 @@ class Linter:
                                 f"Rule {crawler.code} returned conflicting "
                                 "fixes with the same anchor. This is only "
                                 "supported for create_before+create_after, so "
-                                f"the fixes will not be applied. {fixes!r}"
+                                "the fixes will not be applied. "
                             )
+                            for uuid, info in anchor_info.items():
+                                if not info.is_valid:
+                                    message += f"\n{uuid}:"
+                                    for fix in info.fixes:
+                                        message += f"\n    {fix}"
                             cls._report_conflicting_fixes_same_anchor(message)
                             for lint_result in linting_errors:
                                 lint_result.fixes = []
@@ -636,6 +656,11 @@ class Linter:
                                 # we want to stop.
                                 cls._warn_unfixable(crawler.code)
 
+                    # Record rule timing
+                    rule_timings.append(
+                        (crawler.code, crawler.name, time.monotonic() - t0)
+                    )
+
                 if fix and not changed:
                     # We did not change the file. Either the file is clean (no
                     # fixes), or any fixes which are present will take us back
@@ -671,18 +696,21 @@ class Linter:
                     # Reason: When the linter hits the loop limit, the file is often
                     # messy, e.g. some of the fixes were applied repeatedly, possibly
                     # other weird things. We don't want the user to see this junk!
-                    return save_tree, initial_linting_errors, ignore_buff
+                    return save_tree, initial_linting_errors, ignore_buff, rule_timings
 
         if config.get("ignore_templated_areas", default=True):
             initial_linting_errors = cls.remove_templated_errors(initial_linting_errors)
 
-        return tree, initial_linting_errors, ignore_buff
+        linter_logger.info("\n###\n#\n# {}\n#\n###".format("Fixed Tree:"))
+        linter_logger.info("\n" + tree.stringify())
+
+        return tree, initial_linting_errors, ignore_buff, rule_timings
 
     @classmethod
     def lint_parsed(
         cls,
         parsed: ParsedString,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         formatter: Any = None,
         encoding: str = "utf8",
@@ -694,10 +722,15 @@ class Linter:
         if parsed.tree:
             t0 = time.monotonic()
             linter_logger.info("LINTING (%s)", parsed.fname)
-            tree, initial_linting_errors, ignore_buff = cls.lint_fix_parsed(
+            (
+                tree,
+                initial_linting_errors,
+                ignore_buff,
+                rule_timings,
+            ) = cls.lint_fix_parsed(
                 parsed.tree,
                 config=parsed.config,
-                rule_set=rule_set,
+                rule_pack=rule_pack,
                 fix=fix,
                 fname=parsed.fname,
                 templated_file=parsed.templated_file,
@@ -713,6 +746,7 @@ class Linter:
             # If no parsed tree, set to None
             tree = None
             ignore_buff = []
+            rule_timings = []
             if not parsed.config.get("disable_noqa"):
                 # Templating and/or parsing have failed. Look for "noqa"
                 # comments (the normal path for identifying these comments
@@ -725,7 +759,7 @@ class Linter:
                         for lm in parsed.config.get("dialect_obj").lexer_matchers
                         if lm.name == "inline_comment"
                     ][0],
-                    [r.code for r in rule_set],
+                    rule_pack.reference_map,
                 )
                 violations += ignore_violations
 
@@ -738,7 +772,7 @@ class Linter:
             parsed.fname,
             # Deduplicate violations
             LintedFile.deduplicate_in_source_space(violations),
-            time_dict,
+            FileTimings(time_dict, rule_timings),
             tree,
             ignore_mask=ignore_buff,
             templated_file=parsed.templated_file,
@@ -764,7 +798,7 @@ class Linter:
     def lint_rendered(
         cls,
         rendered: RenderedFile,
-        rule_set: List[BaseRule],
+        rule_pack: RulePack,
         fix: bool = False,
         formatter: Any = None,
     ) -> LintedFile:
@@ -772,7 +806,7 @@ class Linter:
         parsed = cls.parse_rendered(rendered)
         return cls.lint_parsed(
             parsed,
-            rule_set=rule_set,
+            rule_pack=rule_pack,
             fix=fix,
             formatter=formatter,
             encoding=rendered.encoding,
@@ -821,7 +855,7 @@ class Linter:
             templated_file = None
             templater_violations = []
 
-        if not templated_file:
+        if templated_file is None:
             linter_logger.info("TEMPLATING FAILED: %s", templater_violations)
 
         # Record time
@@ -882,11 +916,11 @@ class Linter:
     ) -> Tuple[BaseSegment, List[SQLBaseError]]:
         """Return the fixed tree and violations from lintfix when we're fixing."""
         config = config or self.config
-        rule_set = self.get_ruleset(config=config)
-        fixed_tree, violations, _ = self.lint_fix_parsed(
+        rule_pack = self.get_rulepack(config=config)
+        fixed_tree, violations, _, _ = self.lint_fix_parsed(
             tree,
             config,
-            rule_set,
+            rule_pack,
             fix=True,
             fname=fname,
             templated_file=templated_file,
@@ -903,11 +937,11 @@ class Linter:
     ) -> List[SQLBaseError]:
         """Return just the violations from lintfix when we're only linting."""
         config = config or self.config
-        rule_set = self.get_ruleset(config=config)
-        _, violations, _ = self.lint_fix_parsed(
+        rule_pack = self.get_rulepack(config=config)
+        _, violations, _, _ = self.lint_fix_parsed(
             tree,
             config,
-            rule_set,
+            rule_pack,
             fix=False,
             fname=fname,
             templated_file=templated_file,
@@ -938,11 +972,11 @@ class Linter:
             config=config,
         )
         # Get rules as appropriate
-        rule_set = self.get_ruleset(config=config)
+        rule_pack = self.get_rulepack(config=config)
         # Lint the file and return the LintedFile
         return self.lint_parsed(
             parsed,
-            rule_set,
+            rule_pack,
             fix=fix,
             formatter=self.formatter,
             encoding=encoding,
@@ -1046,7 +1080,12 @@ class Linter:
             for ignore_base, ignore_spec in ignores.items():
                 abs_ignore_base = os.path.abspath(ignore_base)
                 if abs_fpath.startswith(
-                    abs_ignore_base + os.sep
+                    abs_ignore_base
+                    + (
+                        ""
+                        if os.path.dirname(abs_ignore_base) == abs_ignore_base
+                        else os.sep
+                    )
                 ) and ignore_spec.match_file(
                     os.path.relpath(abs_fpath, abs_ignore_base)
                 ):
@@ -1103,17 +1142,42 @@ class Linter:
         processes: Optional[int] = None,
     ) -> LintedDir:
         """Lint a path."""
-        linted_path = LintedDir(path)
-        if self.formatter:
-            self.formatter.dispatch_path(path)
-        fnames = list(
-            self.paths_from_path(
+        return self.lint_paths(
+            (path,), fix, ignore_non_existent_files, ignore_files, processes
+        ).paths[0]
+
+    def lint_paths(
+        self,
+        paths: Tuple[str, ...],
+        fix: bool = False,
+        ignore_non_existent_files: bool = False,
+        ignore_files: bool = True,
+        processes: Optional[int] = None,
+        apply_fixes: bool = False,
+        fixed_file_suffix: str = "",
+        fix_even_unparsable: bool = False,
+    ) -> LintingResult:
+        """Lint an iterable of paths."""
+        # If no paths specified - assume local
+        if not paths:  # pragma: no cover
+            paths = (os.getcwd(),)
+        # Set up the result to hold what we get back
+        result = LintingResult()
+
+        expanded_paths: List[str] = []
+        expanded_path_to_linted_dir = {}
+        for path in paths:
+            linted_dir = LintedDir(path)
+            result.add(linted_dir)
+            for fname in self.paths_from_path(
                 path,
                 ignore_non_existent_files=ignore_non_existent_files,
                 ignore_files=ignore_files,
-            )
-        )
+            ):
+                expanded_paths.append(fname)
+                expanded_path_to_linted_dir[fname] = linted_dir
 
+        files_count = len(expanded_paths)
         if processes is None:
             processes = self.config.get("processes", default=1)
 
@@ -1131,72 +1195,41 @@ class Linter:
             self.formatter.dispatch_processing_header(effective_processes)
 
         # Show files progress bar only when there is more than one.
-        files_count = len(fnames)
+        first_path = expanded_paths[0] if expanded_paths else ""
         progress_bar_files = tqdm(
             total=files_count,
-            desc=f"file {os.path.basename(fnames[0] if fnames else '')}",
+            desc=f"file {first_path}",
             leave=False,
             disable=files_count <= 1 or progress_bar_configuration.disable_progress_bar,
         )
 
-        for i, linted_file in enumerate(runner.run(fnames, fix), start=1):
-            linted_path.add(linted_file)
+        for i, linted_file in enumerate(runner.run(expanded_paths, fix), start=1):
+            linted_dir = expanded_path_to_linted_dir[linted_file.path]
+            linted_dir.add(linted_file)
             # If any fatal errors, then stop iteration.
             if any(v.fatal for v in linted_file.violations):  # pragma: no cover
                 linter_logger.error("Fatal linting error. Halting further linting.")
                 break
 
+            # If we're applying fixes, then do that here.
+            if apply_fixes:
+                num_tmp_prs_errors = linted_file.num_violations(
+                    types=TMP_PRS_ERROR_TYPES,
+                    filter_ignore=False,
+                    filter_warning=False,
+                )
+                if fix_even_unparsable or num_tmp_prs_errors == 0:
+                    linted_file.persist_tree(
+                        suffix=fixed_file_suffix, formatter=self.formatter
+                    )
+
             # Progress bar for files is rendered only when there is more than one file.
-            # Additionally as it's updated after each loop, we need to get file name
+            # Additionally, as it's updated after each loop, we need to get file name
             # from the next loop. This is why `enumerate` starts with `1` and there
             # is `i < len` to not exceed files list length.
             progress_bar_files.update(n=1)
-            if i < len(fnames):
-                progress_bar_files.set_description(
-                    f"file {os.path.basename(fnames[i])}"
-                )
-
-        return linted_path
-
-    def lint_paths(
-        self,
-        paths: Tuple[str, ...],
-        fix: bool = False,
-        ignore_non_existent_files: bool = False,
-        ignore_files: bool = True,
-        processes: Optional[int] = None,
-    ) -> LintingResult:
-        """Lint an iterable of paths."""
-        paths_count = len(paths)
-
-        # If no paths specified - assume local
-        if not paths_count:  # pragma: no cover
-            paths = (os.getcwd(),)
-        # Set up the result to hold what we get back
-        result = LintingResult()
-
-        progress_bar_paths = tqdm(
-            total=paths_count,
-            desc="path",
-            leave=False,
-            disable=paths_count <= 1 or progress_bar_configuration.disable_progress_bar,
-        )
-        for path in paths:
-            progress_bar_paths.set_description(f"path {path}")
-
-            # Iterate through files recursively in the specified directory (if it's a
-            # directory) or read the file directly if it's not
-            result.add(
-                self.lint_path(
-                    path,
-                    fix=fix,
-                    ignore_non_existent_files=ignore_non_existent_files,
-                    ignore_files=ignore_files,
-                    processes=processes,
-                )
-            )
-
-            progress_bar_paths.update(1)
+            if i < len(expanded_paths):
+                progress_bar_files.set_description(f"file {expanded_paths[i]}")
 
         result.stop_timer()
         return result
