@@ -1,5 +1,6 @@
 """Defines the linter class."""
 
+import csv
 import time
 from typing import (
     Any,
@@ -8,6 +9,8 @@ from typing import (
     Optional,
     overload,
     Tuple,
+    Union,
+    Set,
 )
 from typing_extensions import Literal
 
@@ -15,16 +18,14 @@ from sqlfluff.cli import EXIT_FAIL, EXIT_SUCCESS
 
 from sqlfluff.core.errors import (
     CheckTuple,
-    SQLLintError,
-    SQLParseError,
-    SQLTemplaterError,
 )
 
-from sqlfluff.core.timing import TimingSummary
+from sqlfluff.core.timing import TimingSummary, RuleTimingSummary
 
 # Classes needed only for type checking
 from sqlfluff.core.parser.segments.base import BaseSegment
 from sqlfluff.core.linter.linted_dir import LintedDir
+from sqlfluff.core.linter.linted_file import TMP_PRS_ERROR_TYPES
 
 
 class LintingResult:
@@ -80,7 +81,9 @@ class LintingResult:
         """Default overload method."""
         ...
 
-    def check_tuples(self, by_path=False):
+    def check_tuples(
+        self, by_path=False
+    ) -> Union[List[CheckTuple], Dict[LintedDir, List[CheckTuple]]]:
         """Fetch all check_tuples from all contained `LintedDir` objects.
 
         Args:
@@ -138,13 +141,77 @@ class LintingResult:
         all_stats["status"] = "FAIL" if all_stats["violations"] > 0 else "PASS"
         return all_stats
 
-    def timing_summary(self) -> Dict[str, Dict[str, float]]:
+    def timing_summary(self) -> Dict[str, Dict[str, Any]]:
         """Return a timing summary."""
         timing = TimingSummary()
+        rules_timing = RuleTimingSummary()
         for dir in self.paths:
             for file in dir.files:
-                timing.add(file.time_dict)
-        return timing.summary()
+                if file.timings:
+                    timing.add(file.timings.step_timings)
+                    rules_timing.add(file.timings.rule_timings)
+        return {**timing.summary(), **rules_timing.summary()}
+
+    def persist_timing_records(self, filename: str) -> None:
+        """Persist the timing records as a csv for external analysis."""
+        meta_fields = [
+            "path",
+            "source_chars",
+            "templated_chars",
+            "segments",
+            "raw_segments",
+        ]
+        timing_fields = ["templating", "lexing", "parsing", "linting"]
+
+        # Iterate through all the files to get rule timing information so
+        # we know what headings we're going to need.
+        rule_codes: Set[str] = set()
+        file_timing_dicts: Dict[str, dict] = {}
+        for dir in self.paths:
+            for file in dir.files:
+                if not file.timings:  # pragma: no cover
+                    continue
+                file_timing_dicts[file.path] = file.timings.get_rule_timing_dict()
+                rule_codes.update(file_timing_dicts[file.path].keys())
+
+        with open(filename, "w", newline="") as f:
+            writer = csv.DictWriter(
+                f, fieldnames=meta_fields + timing_fields + sorted(rule_codes)
+            )
+
+            writer.writeheader()
+
+            for dir in self.paths:
+                for file in dir.files:
+                    if not file.timings:  # pragma: no cover
+                        continue
+                    writer.writerow(
+                        {
+                            "path": file.path,
+                            "source_chars": (
+                                len(file.templated_file.source_str)
+                                if file.templated_file
+                                else ""
+                            ),
+                            "templated_chars": (
+                                len(file.templated_file.templated_str)
+                                if file.templated_file
+                                else ""
+                            ),
+                            "segments": (
+                                file.tree.count_segments(raw_only=False)
+                                if file.tree
+                                else ""
+                            ),
+                            "raw_segments": (
+                                file.tree.count_segments(raw_only=True)
+                                if file.tree
+                                else ""
+                            ),
+                            **file.timings.step_timings,
+                            **file_timing_dicts[file.path],
+                        }
+                    )
 
     def as_records(self) -> List[dict]:
         """Return the result as a list of dictionaries.
@@ -168,11 +235,13 @@ class LintingResult:
             if violations
         ]
 
-    def persist_changes(self, formatter, **kwargs) -> dict:
+    def persist_changes(self, formatter, fixed_file_suffix: str = "") -> dict:
         """Run all the fixes for all the files and return a dict."""
         return self.combine_dicts(
             *(
-                path.persist_changes(formatter=formatter, **kwargs)
+                path.persist_changes(
+                    formatter=formatter, fixed_file_suffix=fixed_file_suffix
+                )
                 for path in self.paths
             )
         )
@@ -187,35 +256,29 @@ class LintingResult:
             )
         return self.paths[0].tree
 
-    TMP_PRS_ERROR_TYPES = (SQLTemplaterError, SQLParseError)
-
     def count_tmp_prs_errors(self) -> Tuple[int, int]:
         """Count templating or parse errors before and after filtering."""
         total_errors = self.num_violations(
-            types=self.TMP_PRS_ERROR_TYPES, filter_ignore=False
+            types=TMP_PRS_ERROR_TYPES,
+            filter_ignore=False,
+            filter_warning=False,
         )
         num_filtered_errors = 0
         for linted_dir in self.paths:
             for linted_file in linted_dir.files:
                 num_filtered_errors += linted_file.num_violations(
-                    types=self.TMP_PRS_ERROR_TYPES
+                    types=TMP_PRS_ERROR_TYPES
                 )
         return total_errors, num_filtered_errors
 
     def discard_fixes_for_lint_errors_in_files_with_tmp_or_prs_errors(self) -> None:
         """Discard lint fixes for files with templating or parse errors."""
         total_errors = self.num_violations(
-            types=self.TMP_PRS_ERROR_TYPES, filter_ignore=False
+            types=TMP_PRS_ERROR_TYPES,
+            filter_ignore=False,
+            filter_warning=False,
         )
         if total_errors:
             for linted_dir in self.paths:
                 for linted_file in linted_dir.files:
-                    num_errors = linted_file.num_violations(
-                        types=self.TMP_PRS_ERROR_TYPES, filter_ignore=False
-                    )
-                    if num_errors:
-                        # File has errors. Discard all the SQLLintError fixes:
-                        # they are potentially unsafe.
-                        for violation in linted_file.violations:
-                            if isinstance(violation, SQLLintError):
-                                violation.fixes = []
+                    linted_file.discard_fixes_if_tmp_or_prs_errors()
