@@ -13,7 +13,7 @@ from collections.abc import MutableSet
 from copy import deepcopy, copy
 from dataclasses import dataclass, field, replace
 from io import StringIO
-from itertools import takewhile, chain
+from itertools import chain
 from typing import (
     Any,
     Callable,
@@ -86,11 +86,13 @@ class PathStep:
         segment (:obj:`BaseSegment`): The segment in the chain.
         idx (int): The index of the target within its `segment`.
         len (int): The number of children `segment` has.
+        code_idxs (:obj:`tuple` of int): The indices which contain code.
     """
 
     segment: "BaseSegment"
     idx: int
     len: int
+    code_idxs: Tuple[int, ...]
 
 
 @dataclass
@@ -111,35 +113,6 @@ class FixPatch:
         """Generate a tuple of this fix for deduping."""
         return (self.source_slice, self.fixed_raw)
 
-    @classmethod
-    def infer_from_template(
-        cls,
-        templated_slice: slice,
-        fixed_raw: str,
-        patch_category: str,
-        templated_file: TemplatedFile,
-    ):
-        """Infer source position from just templated position.
-
-        In cases where we expect it to be uncontroversial it
-        is sometimes more straightforward to just leverage
-        the existing mapping functions to auto-generate the
-        source position rather than calculating it explicitly.
-        """
-        # NOTE: There used to be error handling here to catch ValueErrors.
-        # Removed in July 2022 because untestable.
-        source_slice = templated_file.templated_slice_to_source_slice(
-            templated_slice,
-        )
-        return cls(
-            source_slice=source_slice,
-            templated_slice=templated_slice,
-            patch_category=patch_category,
-            fixed_raw=fixed_raw,
-            templated_str=templated_file.templated_str[templated_slice],
-            source_str=templated_file.source_str[source_slice],
-        )
-
 
 @dataclass
 class AnchorEditInfo:
@@ -160,6 +133,10 @@ class AnchorEditInfo:
         We also allow potentially multiple source fixes on the same
         anchor by condensing them together here.
         """
+        if fix in self.fixes:
+            # Deduplicate fixes in case it's already in there.
+            return
+
         if fix.is_just_source_edit():
             assert fix.edit
             # is_just_source_edit confirms there will be a list
@@ -232,6 +209,10 @@ class SegmentMetaclass(type):
         here saves calculating it at runtime for each
         instance of the class.
         """
+        # Create a cache uuid on definition.
+        # We do it here so every _definition_ of a segment
+        # gets a unique UUID regardless of dialect.
+        class_dict["_cache_key"] = uuid4().hex
         class_obj = super().__new__(mcs, name, bases, class_dict)
         added_type = class_dict.get("type", None)
         class_types = {added_type} if added_type else set()
@@ -278,6 +259,10 @@ class BaseSegment(metaclass=SegmentMetaclass):
     # What other kwargs need to be copied when applying fixes.
     additional_kwargs: List[str] = []
     pos_marker: Optional[PositionMarker]
+    # NOTE: Cache key is generated by the SegmentMetaclass
+    _cache_key: str
+    # _preface_modifier used in ._preface()
+    _preface_modifier: str = ""
 
     def __init__(
         self,
@@ -475,9 +460,10 @@ class BaseSegment(metaclass=SegmentMetaclass):
     ) -> List[Tuple["RawSegment", List[PathStep]]]:
         """Returns a list of raw segments in this segment with the ancestors."""
         buffer = []
+        code_idxs = tuple(idx for idx, seg in enumerate(self.segments) if seg.is_code)
         for idx, seg in enumerate(self.segments):
             # If it's a raw, yield it with this segment as the parent
-            new_step = [PathStep(self, idx, len(self.segments))]
+            new_step = [PathStep(self, idx, len(self.segments), code_idxs)]
             if seg.is_type("raw"):
                 buffer.append((seg, new_step))
             # If it's not, recurse - prepending self to the ancestor stack
@@ -518,12 +504,12 @@ class BaseSegment(metaclass=SegmentMetaclass):
         # * Source slice not empty: If it's empty, this means it doesn't appear
         #   in the source, e.g. because it is new code generated by a lint fix.
         #   Return False for these.
-        # * Source string doesn't match raw segment contents. This can only
-        #   happen if templating is involved.
+        # * It's not a literal slice. If it's a literal and has size then it's
+        #   not templated.
         assert self.pos_marker
         return (
             self.pos_marker.source_slice.start != self.pos_marker.source_slice.stop
-            and self.raw != self.pos_marker.source_str()
+            and not self.pos_marker.is_literal()
         )
 
     # ################ STATIC METHODS
@@ -533,8 +519,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Return a tuple structure from an iterable of segments."""
         return tuple(seg.to_tuple(**kwargs) for seg in segs)
 
-    @staticmethod
-    def _suffix():
+    def _suffix(self) -> str:
         """Return any extra output required at the end when logging.
 
         NB Override this for specific subclasses if we want extra output.
@@ -640,30 +625,47 @@ class BaseSegment(metaclass=SegmentMetaclass):
         # and backward.
         segment_buffer: Tuple["BaseSegment", ...] = ()
         for idx, segment in enumerate(segments):
-            repositioned_seg = copy(segment)
+            repositioned_seg = segment.copy()
             # Fill any that don't have a position.
             if not repositioned_seg.pos_marker:
                 # Can we get a position from the previous?
+                start_point = None
                 if idx > 0:
                     prev_seg = segment_buffer[idx - 1]
                     # Given we're going back in the buffer we should
                     # have set the position marker for everything already
                     # in there. This is mostly a hint to mypy.
                     assert prev_seg.pos_marker
-                    repositioned_seg.pos_marker = prev_seg.pos_marker.end_point_marker()
+                    start_point = prev_seg.pos_marker.end_point_marker()
                 # Can we get it from the parent?
                 elif parent_pos:
-                    repositioned_seg.pos_marker = parent_pos.start_point_marker()
-                # Search forward for a following one, if we have to?
-                else:
-                    for fwd_seg in segments[idx + 1 :]:
-                        if fwd_seg.pos_marker:
-                            repositioned_seg.pos_marker = (
-                                fwd_seg.pos_marker.start_point_marker()
-                            )
-                            break
-                    else:  # pragma: no cover
-                        raise ValueError("Unable to position new segment")
+                    start_point = parent_pos.start_point_marker()
+
+                # Search forward for the end point.
+                end_point = None
+                for fwd_seg in segments[idx + 1 :]:
+                    if fwd_seg.pos_marker:
+                        # NOTE: Use raw segments because it's more reliable.
+                        end_point = fwd_seg.raw_segments[
+                            0
+                        ].pos_marker.start_point_marker()
+                        break
+
+                if start_point and end_point and start_point != end_point:
+                    # We should construct a wider position marker.
+                    repositioned_seg.pos_marker = PositionMarker.from_points(
+                        start_point,
+                        end_point,
+                    )
+                # If we have start point (or if they were equal above),
+                # just apply start point.
+                elif start_point:
+                    repositioned_seg.pos_marker = start_point
+                # Do we have an end?
+                elif end_point:
+                    repositioned_seg.pos_marker = end_point
+                else:  # pragma: no cover
+                    raise ValueError("Unable to position new segment")
 
             assert repositioned_seg.pos_marker  # hint for mypy
             # Update the working position.
@@ -690,7 +692,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
     # ################ CLASS METHODS
 
     @classmethod
-    def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
+    def simple(cls, parse_context: ParseContext, crumbs=None):
         """Does this matcher support an uppercase hash matching route?
 
         This should be true if the MATCH grammar is simple. Most more
@@ -704,6 +706,14 @@ class BaseSegment(metaclass=SegmentMetaclass):
             # simple.
             return None
 
+    @classmethod
+    def cache_key(cls) -> str:
+        """Return the cache key for this segment definition.
+
+        NOTE: The key itself is generated on _definition_ by the metaclass.
+        """
+        return cls._cache_key
+
     @classmethod
     def is_optional(cls):
         """Return True if this segment is optional.
@@ -845,7 +855,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Returns the preamble to any logging."""
         padded_type = "{padding}{modifier}{type}".format(
             padding=" " * (ident * tabsize),
-            modifier="[META] " if self.is_meta else "",
+            modifier=self._preface_modifier,
             type=self.get_type() + ":",
         )
         preface = "{pos:20}|{padded_type:60}  {suffix}".format(
@@ -887,7 +897,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
 
         self._recalculate_caches()
 
-    def get_start_point_marker(self):
+    def get_start_point_marker(self):  # pragma: no cover
         """Get a point marker at the start of this segment."""
         return self.pos_marker.start_point_marker()
 
@@ -982,6 +992,10 @@ class BaseSegment(metaclass=SegmentMetaclass):
     def copy(self):
         """Copy the segment recursively, with appropriate copying of references."""
         new_seg = copy(self)
+        # Position markers are immutable, and it's important that we keep
+        # a reference to the same TemplatedFile, so keep the same position
+        # marker.
+        new_seg.pos_marker = self.pos_marker
         if self.segments:
             new_seg.segments = tuple(seg.copy() for seg in self.segments)
         return new_seg
@@ -1006,7 +1020,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
         return [item for s in self.segments for item in s.raw_segments]
 
     def iter_segments(self, expanding=None, pass_through=False):
-        """Iterate raw segments, optionally expanding some children."""
+        """Iterate segments, optionally expanding some children."""
         for s in self.segments:
             if expanding and s.is_type(*expanding):
                 yield from s.iter_segments(
@@ -1110,7 +1124,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
                         no_recursive_seg_type=no_recursive_seg_type,
                     )
 
-    def path_to(self, other) -> List[PathStep]:
+    def path_to(self, other: "BaseSegment") -> List[PathStep]:
         """Given a segment which is assumed within self, get the intermediate segments.
 
         Returns:
@@ -1131,7 +1145,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
             return []  # pragma: no cover
 
         # Are we in the right ballpark?
-        # NB: Comparisons have a higher precedence than `not`.
+        # NOTE: Comparisons have a higher precedence than `not`.
         if not self.get_start_loc() <= other.get_start_loc() <= self.get_end_loc():
             return []
 
@@ -1139,9 +1153,11 @@ class BaseSegment(metaclass=SegmentMetaclass):
         if not self.segments:
             return []
 
+        # Check code idxs
+        code_idxs = tuple(idx for idx, seg in enumerate(self.segments) if seg.is_code)
         # Check through each of the child segments
         for idx, seg in enumerate(self.segments):
-            step = PathStep(self, idx, len(self.segments))
+            step = PathStep(self, idx, len(self.segments), code_idxs)
             # Have we found the target?
             if seg is other:
                 return [step]
@@ -1361,11 +1377,22 @@ class BaseSegment(metaclass=SegmentMetaclass):
 
                                 # We're doing a replacement (it could be a single
                                 # segment or an iterable)
+                                consumed_pos = False
                                 if isinstance(f.edit, BaseSegment):
                                     seg_buffer.append(f.edit)  # pragma: no cover TODO?
                                 else:
                                     for s in f.edit:
                                         seg_buffer.append(s)
+                                        # If one of them has the same raw representation
+                                        # then the first that matches gets to take the
+                                        # original position marker.
+                                        if (
+                                            f.edit_type == "replace"
+                                            and s.raw == seg.raw
+                                            and not consumed_pos
+                                        ):
+                                            seg_buffer[-1].pos_marker = seg.pos_marker
+                                            consumed_pos = True
 
                                 if f.edit_type == "create_before":
                                     # in the case of a creation before, also add this
@@ -1407,46 +1434,19 @@ class BaseSegment(metaclass=SegmentMetaclass):
                 seg_buffer.append(s)
                 seg_buffer.extend(after)
 
-            before = []
-            after = []
-            # If there's a parse grammar and this segment is not allowed to
-            # start or end with non-code, check for (and fix) misplaced
-            # segments. The reason for the parse grammar check is autofix if and
-            # only if parse() would've complained, and it has the same parse
-            # grammar check prior to checking can_start_end_non_code.
-            if r.parse_grammar and not r.can_start_end_non_code:
-                idx_non_code = self._find_start_or_end_non_code(seg_buffer)
-                # Are there misplaced segments from a fix?
-                if idx_non_code is not None:
-                    # Yes. Fix the misplaced segments: Do not include them
-                    # in the new segment's children. Instead, return them to the
-                    # caller, which will place them *adjacent to* the new
-                    # segment, in effect, bubbling them up to the tree to a
-                    # valid location.
-                    save_seg_buffer = list(seg_buffer)
-                    before.extend(
-                        takewhile(
-                            lambda seg: not self._is_code_or_meta(seg), seg_buffer
-                        )
-                    )
-                    seg_buffer = seg_buffer[len(before) :]
-                    after.extend(
-                        takewhile(
-                            lambda seg: not self._is_code_or_meta(seg),
-                            reversed(seg_buffer),
-                        )
-                    )
-                    after.reverse()
-                    seg_buffer = seg_buffer[: len(seg_buffer) - len(after)]
-                    assert before + seg_buffer + after == save_seg_buffer
-                    linter_logger.debug(
-                        "After applying fixes, segment %s violated "
-                        "'can_start_end_non_code=False' constraint. Autofixing, "
-                        "before=%s, after=%s",
-                        self,
-                        before,
-                        after,
-                    )
+            # After fixing we should be able to rely on whitespace being
+            # inserted in appropriate places. That logic now lives in
+            # `BaseRule._choose_anchor_segment()`, rather than here.
+
+            # Rather than fix that here, we simply assert that it has been
+            # done. This will raise issues in testing, but shouldn't in use.
+            if r.parse_grammar and not r.can_start_end_non_code and seg_buffer:
+                assert not self._find_start_or_end_non_code(seg_buffer), (
+                    "Found inappropriate fix application: inappropriate "
+                    "whitespace positioning. Post `_choose_anchor_segment`. "
+                    "Please report this issue on GitHub with your SQL query. "
+                )
+
             # Reform into a new segment
             r = r.__class__(
                 # Realign the segments within
@@ -1560,11 +1560,15 @@ class BaseSegment(metaclass=SegmentMetaclass):
             # First yield any source fixes
             yield from self._iter_source_fix_patches(templated_file)
             # Then yield the position in the source file and the patch
-            yield FixPatch.infer_from_template(
-                self.pos_marker.templated_slice,
-                self.raw,
+            yield FixPatch(
+                source_slice=self.pos_marker.source_slice,
+                templated_slice=self.pos_marker.templated_slice,
                 patch_category="literal",
-                templated_file=templated_file,
+                fixed_raw=self.raw,
+                templated_str=templated_file.templated_str[
+                    self.pos_marker.templated_slice
+                ],
+                source_str=templated_file.source_str[self.pos_marker.source_slice],
             )
         # Can we go deeper?
         elif not self.segments:
@@ -1603,6 +1607,7 @@ class BaseSegment(metaclass=SegmentMetaclass):
 
                 # If we get here, then we know it's an original. Check for deletions at
                 # the point before this segment (vs the TEMPLATED).
+                # Deletions in this sense could also mean source consumption.
                 start_diff = segment.pos_marker.templated_slice.start - templated_idx
 
                 # Check to see whether there's a discontinuity before the current
@@ -1610,15 +1615,29 @@ class BaseSegment(metaclass=SegmentMetaclass):
                 if start_diff > 0 or insert_buff:
                     # If we have an insert buffer, then it's an edit, otherwise a
                     # deletion.
-                    yield FixPatch.infer_from_template(
-                        slice(
-                            segment.pos_marker.templated_slice.start
-                            - max(start_diff, 0),
-                            segment.pos_marker.templated_slice.start,
+
+                    # For the start of the next segment, we need the position of the
+                    # first raw, not the pos marker of the whole thing. That accounts
+                    # better for loops.
+                    first_segment_pos = segment.raw_segments[0].pos_marker
+                    yield FixPatch(
+                        # Whether the source slice is zero depends on the start_diff.
+                        # A non-zero start diff implies a deletion, or more likely
+                        # a consumed element of the source. We can use the tracking
+                        # markers from the last segment to recreate where this element
+                        # should be inserted in both source and template.
+                        source_slice=slice(
+                            source_idx,
+                            first_segment_pos.source_slice.start,
+                        ),
+                        templated_slice=slice(
+                            templated_idx,
+                            first_segment_pos.templated_slice.start,
                         ),
-                        insert_buff,
                         patch_category="mid_point",
-                        templated_file=templated_file,
+                        fixed_raw=insert_buff,
+                        templated_str="",
+                        source_str="",
                     )
 
                     insert_buff = ""
@@ -1691,7 +1710,7 @@ class BracketedSegment(BaseSegment):
         super().__init__(*args, **kwargs)
 
     @classmethod
-    def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
+    def simple(cls, parse_context: ParseContext, crumbs=None):
         """Simple methods for bracketed and the persistent brackets."""
         start_brackets = [
             start_bracket
@@ -1700,12 +1719,17 @@ class BracketedSegment(BaseSegment):
             )
             if persistent
         ]
-        start_simple = []
+        simple_raws = set()
         for ref in start_brackets:
-            start_simple += parse_context.dialect.ref(ref).simple(
+            bracket_simple = parse_context.dialect.ref(ref).simple(
                 parse_context, crumbs=crumbs
             )
-        return start_simple
+            assert bracket_simple, "All bracket segments must support simple."
+            assert bracket_simple[0], "All bracket segments must support raw simple."
+            # NOTE: By making this assumption we don't have to handle the "typed"
+            # simple here.
+            simple_raws.update(bracket_simple[0])
+        return frozenset(simple_raws), frozenset()
 
     @classmethod
     def match(
