@@ -76,7 +76,15 @@ class JinjaTracer:
         trace_template = self.make_template(trace_template_str)
         trace_template_output = trace_template.render()
         # Split output by section. Each section has two possible formats.
-        trace_entries = list(regex.finditer(r"\0", trace_template_output))
+        trace_entries: List[regex.Match] = list(
+            regex.finditer(r"\0", trace_template_output)
+        )
+        # If the file has no templated entries, we should just iterate
+        # through the raw slices to add all the placeholders.
+        if not trace_entries:
+            for raw_idx, _ in enumerate(self.raw_sliced):
+                self.record_trace(0, raw_idx)
+
         for match_idx, match in enumerate(trace_entries):
             pos1 = match.span()[0]
             try:
@@ -184,7 +192,7 @@ class JinjaTracer:
                 slice(self.source_idx, self.source_idx + target_slice_length),
             )
         )
-        if slice_type in ("literal", "templated"):
+        if target_slice_length:
             self.source_idx += target_slice_length
 
 
@@ -454,7 +462,6 @@ class JinjaAnalyzer:
                     slice_idx = len(self.raw_sliced) - 1
                     self.idx_raw += len(str_buff)
                 if block_type.startswith("block"):
-                    self.track_block_start(block_type, tag_contents[0])
                     self.track_block_end(block_type, tag_contents[0])
                     self.update_next_slice_indices(
                         slice_idx, block_type, tag_contents[0]
@@ -522,7 +529,8 @@ class JinjaAnalyzer:
         # :TRICKY: Syntactically, the Jinja {% include %} directive looks like
         # a block, but its behavior is basically syntactic sugar for
         # {{ open("somefile).read() }}. Thus, treat it as templated code.
-        if tag_name == "include":
+        # It's a similar situation with {% import %} and {% from ... import %}.
+        if tag_name in ["include", "import", "from", "do"]:
             block_type = "templated"
         elif tag_name.startswith("end"):
             block_type = "block_end"
@@ -559,20 +567,6 @@ class JinjaAnalyzer:
             trimmed_parts = trimmed_content.split()
         return trimmed_parts
 
-    def track_block_start(self, block_type: str, tag_name: str) -> None:
-        """On starting a 'call' block, set slice_type to "templated"."""
-        if block_type == "block_start" and tag_name == "call":
-            # Replace RawSliceInfo for this slice with one that has block_type
-            # "templated".
-            old_raw_file_slice = self.raw_sliced[-1]
-            self.raw_sliced[-1] = old_raw_file_slice._replace(slice_type="templated")
-
-            # Move existing raw_slice_info entry since it's keyed by RawFileSlice.
-            self.raw_slice_info[self.raw_sliced[-1]] = self.raw_slice_info[
-                old_raw_file_slice
-            ]
-            del self.raw_slice_info[old_raw_file_slice]
-
     def track_block_end(self, block_type: str, tag_name: str) -> None:
         """On ending a 'for' or 'if' block, set up tracking."""
         if block_type == "block_end" and tag_name in (
