@@ -28,7 +28,6 @@ from sqlfluff.core.parser import (
     RegexParser,
     SegmentGenerator,
     Sequence,
-    StartsWith,
     StringLexer,
     StringParser,
     SymbolSegment,
@@ -131,6 +130,14 @@ bigquery_dialect.add(
         ansi.IdentifierSegment,
         type="naked_identifier_all",
     ),
+    NakedIdentifierPart=RegexParser(
+        # The part of a an identifier after a hyphen.
+        # NOTE: This one can match an "all numbers" variant.
+        # https://cloud.google.com/resource-manager/docs/creating-managing-projects
+        r"[A-Z0-9_]+",
+        ansi.IdentifierSegment,
+        type="naked_identifier",
+    ),
     SingleIdentifierFullGrammar=OneOf(
         Ref("NakedIdentifierSegment"),
         Ref("QuotedIdentifierSegment"),
@@ -146,6 +153,9 @@ bigquery_dialect.add(
             Ref("ArrayLiteralSegment"),
             Ref("TupleSegment"),
             Ref("BaseExpressionElementGrammar"),
+            terminators=[
+                Ref("SemicolonSegment"),
+            ],
         ),
     ),
     ExtendedDatetimeUnitSegment=SegmentGenerator(
@@ -203,14 +213,6 @@ bigquery_dialect.replace(
         Ref("NamedArgumentSegment"),
     ),
     TrimParametersGrammar=Nothing(),
-    SimpleArrayTypeGrammar=Sequence(
-        "ARRAY",
-        Bracketed(
-            Ref("DatatypeSegment"),
-            bracket_type="angle",
-            bracket_pairs_set="angle_bracket_pairs",
-        ),
-    ),
     # BigQuery allows underscore in parameter names, and also anything if quoted in
     # backticks
     ParameterNameSegment=OneOf(
@@ -245,13 +247,13 @@ bigquery_dialect.replace(
 
 # Set Keywords
 bigquery_dialect.sets("unreserved_keywords").clear()
-bigquery_dialect.sets("unreserved_keywords").update(
-    [n.strip().upper() for n in bigquery_unreserved_keywords.split("\n")]
+bigquery_dialect.update_keywords_set_from_multiline_string(
+    "unreserved_keywords", bigquery_unreserved_keywords
 )
 
 bigquery_dialect.sets("reserved_keywords").clear()
-bigquery_dialect.sets("reserved_keywords").update(
-    [n.strip().upper() for n in bigquery_reserved_keywords.split("\n")]
+bigquery_dialect.update_keywords_set_from_multiline_string(
+    "reserved_keywords", bigquery_reserved_keywords
 )
 
 # Add additional datetime units
@@ -311,17 +313,25 @@ bigquery_dialect.sets("angle_bracket_pairs").update(
 )
 
 
+class ArrayTypeSegment(ansi.ArrayTypeSegment):
+    """Prefix for array literals specifying the type."""
+
+    type = "array_type"
+    match_grammar = Sequence(
+        "ARRAY",
+        Bracketed(
+            Ref("DatatypeSegment"),
+            bracket_type="angle",
+            bracket_pairs_set="angle_bracket_pairs",
+        ),
+    )
+
+
 class QualifyClauseSegment(BaseSegment):
     """A `QUALIFY` clause like in `SELECT`."""
 
     type = "qualify_clause"
-    match_grammar = StartsWith(
-        "QUALIFY",
-        terminator=OneOf("WINDOW", "ORDER", "LIMIT"),
-        enforce_whitespace_preceding_terminator=True,
-    )
-
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "QUALIFY",
         Indent,
         OptionallyBracketed(Ref("ExpressionSegment")),
@@ -343,6 +353,30 @@ class SetOperatorSegment(BaseSegment):
     )
 
 
+class SetExpressionSegment(ansi.SetExpressionSegment):
+    """A set expression with either Union, Minus, Except or Intersect."""
+
+    match_grammar: Matchable = Sequence(
+        OneOf(
+            Ref("NonSetSelectableGrammar"),
+            Bracketed(Ref("SetExpressionSegment")),
+        ),
+        AnyNumberOf(
+            Sequence(
+                Ref("SetOperatorSegment"),
+                OneOf(
+                    Ref("NonSetSelectableGrammar"),
+                    Bracketed(Ref("SetExpressionSegment")),
+                ),
+            ),
+            min_times=1,
+        ),
+        Ref("OrderByClauseSegment", optional=True),
+        Ref("LimitClauseSegment", optional=True),
+        Ref("NamedWindowSegment", optional=True),
+    )
+
+
 class SelectStatementSegment(ansi.SelectStatementSegment):
     """Enhance `SELECT` statement to include QUALIFY."""
 
@@ -422,6 +456,7 @@ class StatementSegment(ansi.StatementSegment):
             Ref("LeaveStatementSegment"),
             Ref("ContinueStatementSegment"),
             Ref("RaiseStatementSegment"),
+            Ref("AlterViewStatementSegment"),
             Ref("CreateMaterializedViewStatementSegment"),
             Ref("AlterMaterializedViewStatementSegment"),
             Ref("DropMaterializedViewStatementSegment"),
@@ -439,6 +474,14 @@ class AssertStatementSegment(BaseSegment):
     match_grammar: Matchable = Sequence(
         "ASSERT",
         Ref("ExpressionSegment"),
+        Sequence(
+            "AS",
+            OneOf(
+                Ref("SingleQuotedLiteralSegment"),
+                Ref("DoubleQuotedLiteralSegment"),
+            ),
+            optional=True,
+        ),
     )
 
 
@@ -468,10 +511,7 @@ class ForInStatementSegment(BaseSegment):
     """
 
     type = "for_in_statement"
-    match_grammar = StartsWith(
-        "FOR", terminator=Sequence("END", "FOR"), include_terminator=True
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "FOR",
         Ref("SingleIdentifierGrammar"),
         "IN",
@@ -513,10 +553,7 @@ class RepeatStatementSegment(BaseSegment):
     """
 
     type = "repeat_statement"
-    match_grammar = StartsWith(
-        "REPEAT", terminator=Sequence("END", "REPEAT"), include_terminator=True
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "REPEAT",
         Indent,
         Ref("RepeatStatementsSegment"),
@@ -554,10 +591,7 @@ class IfStatementSegment(BaseSegment):
     """
 
     type = "if_statement"
-    match_grammar = StartsWith(
-        "IF", terminator=Sequence("END", "IF"), include_terminator=True
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "IF",
         Ref("ExpressionSegment"),
         "THEN",
@@ -612,10 +646,7 @@ class LoopStatementSegment(BaseSegment):
     """
 
     type = "loop_statement"
-    match_grammar = StartsWith(
-        "LOOP", terminator=Sequence("END", "LOOP"), include_terminator=True
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "LOOP",
         Indent,
         Ref("LoopStatementsSegment"),
@@ -648,10 +679,7 @@ class WhileStatementSegment(BaseSegment):
     """
 
     type = "while_statement"
-    match_grammar = StartsWith(
-        "WHILE", terminator=Sequence("END", "WHILE"), include_terminator=True
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "WHILE",
         Ref("ExpressionSegment"),
         "DO",
@@ -668,8 +696,8 @@ class SelectClauseModifierSegment(ansi.SelectClauseModifierSegment):
 
     match_grammar = Sequence(
         # https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax
-        Sequence("AS", OneOf("STRUCT", "VALUE"), optional=True),
         OneOf("DISTINCT", "ALL", optional=True),
+        Sequence("AS", OneOf("STRUCT", "VALUE"), optional=True),
     )
 
 
@@ -795,6 +823,35 @@ class NormalizeFunctionNameSegment(BaseSegment):
     )
 
 
+class FunctionNameSegment(ansi.FunctionNameSegment):
+    """Describes the name of a function.
+
+    This includes any prefix bits, e.g. project, schema or the SAFE keyword.
+    """
+
+    match_grammar: Matchable = Sequence(
+        # Project name, schema identifier, etc.
+        AnyNumberOf(
+            Sequence(
+                # BigQuery Function names can be prefixed by the keyword SAFE to
+                # return NULL instead of error.
+                # https://cloud.google.com/bigquery/docs/reference/standard-sql/functions-reference#safe_prefix
+                OneOf("SAFE", Ref("SingleIdentifierGrammar")),
+                Ref("DotSegment"),
+            ),
+        ),
+        # Base function name
+        OneOf(
+            Ref("FunctionNameIdentifierSegment"),
+            Ref("QuotedIdentifierSegment"),
+        ),
+        # BigQuery allows whitespaces between the `.` of a function refrence or
+        # SAFE prefix. Keeping the explicit `allow_gaps=True` here to
+        # make the distinction from `ansi.FunctionNameSegment` clear.
+        allow_gaps=True,
+    )
+
+
 class FunctionSegment(ansi.FunctionSegment):
     """A scalar or aggregate function.
 
@@ -805,10 +862,6 @@ class FunctionSegment(ansi.FunctionSegment):
     """
 
     match_grammar = Sequence(
-        # BigQuery Function names can be prefixed by the keyword SAFE to
-        # return NULL instead of error.
-        # https://cloud.google.com/bigquery/docs/reference/standard-sql/functions-reference#safe_prefix
-        Sequence("SAFE", Ref("DotSegment"), optional=True),
         OneOf(
             Sequence(
                 # BigQuery EXTRACT allows optional TimeZone
@@ -880,20 +933,7 @@ class FunctionSegment(ansi.FunctionSegment):
                 # Functions returning STRUCTs in BigQuery can have the fields
                 # elements referenced (e.g. ".a"), including wildcards (e.g. ".*")
                 # or multiple nested fields (e.g. ".a.b", or ".a.b.c")
-                Sequence(
-                    Ref("DotSegment"),
-                    AnyNumberOf(
-                        Sequence(
-                            Ref("ParameterNameSegment"),
-                            Ref("DotSegment"),
-                        ),
-                    ),
-                    OneOf(
-                        Ref("ParameterNameSegment"),
-                        Ref("StarSegment"),
-                    ),
-                    optional=True,
-                ),
+                Ref("SemiStructuredAccessorSegment", optional=True),
                 Ref("PostFunctionGrammar", optional=True),
             ),
         ),
@@ -992,10 +1032,10 @@ class DatatypeSegment(ansi.DatatypeSegment):
         Sequence(
             Ref("DatatypeIdentifierSegment"),  # Simple type
             # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#parameterized_data_types
-            Bracketed(Delimited(Ref("NumericLiteralSegment")), optional=True),
+            Ref("BracketedArguments", optional=True),
         ),
         Sequence("ANY", "TYPE"),  # SQL UDFs can specify this "type"
-        Ref("SimpleArrayTypeGrammar"),
+        Ref("ArrayTypeSegment"),
         Ref("StructTypeSegment"),
     )
 
@@ -1005,48 +1045,37 @@ class StructTypeSegment(ansi.StructTypeSegment):
 
     match_grammar = Sequence(
         "STRUCT",
-        Bracketed(
-            Delimited(  # Comma-separated list of field names/types
-                Sequence(
-                    OneOf(
-                        # ParameterNames can look like Datatypes so can't use
-                        # Optional=True here and instead do a OneOf in order
-                        # with DataType only first, followed by both.
+        Ref("StructTypeSchemaSegment", optional=True),
+    )
+
+
+class StructTypeSchemaSegment(BaseSegment):
+    """Expression to construct the schema of a STRUCT datatype."""
+
+    type = "struct_type_schema"
+    match_grammar = Bracketed(
+        Delimited(  # Comma-separated list of field names/types
+            Sequence(
+                OneOf(
+                    # ParameterNames can look like Datatypes so can't use
+                    # Optional=True here and instead do a OneOf in order
+                    # with DataType only first, followed by both.
+                    Ref("DatatypeSegment"),
+                    Sequence(
+                        Ref("ParameterNameSegment"),
                         Ref("DatatypeSegment"),
-                        Sequence(
-                            Ref("ParameterNameSegment"),
-                            Ref("DatatypeSegment"),
-                        ),
                     ),
-                    Ref("OptionsSegment", optional=True),
                 ),
+                AnyNumberOf(Ref("ColumnConstraintSegment")),
+                Ref("OptionsSegment", optional=True),
             ),
-            bracket_type="angle",
-            bracket_pairs_set="angle_bracket_pairs",
         ),
+        bracket_type="angle",
+        bracket_pairs_set="angle_bracket_pairs",
     )
 
 
-class TypelessStructSegment(ansi.TypelessStructSegment):
-    """Expression to construct a STRUCT with implicit types.
-
-    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#typeless_struct_syntax
-    """
-
-    match_grammar = Sequence(
-        "STRUCT",
-        Bracketed(
-            Delimited(
-                Sequence(
-                    Ref("BaseExpressionElementGrammar"),
-                    Ref("AliasExpressionSegment", optional=True),
-                ),
-            ),
-        ),
-    )
-
-
-class TypelessArraySegment(ansi.TypelessArraySegment):
+class ArrayExpressionSegment(ansi.ArrayExpressionSegment):
     """Expression to construct a ARRAY from a subquery.
 
     https://cloud.google.com/bigquery/docs/reference/standard-sql/array_functions#array
@@ -1097,17 +1126,18 @@ class SemiStructuredAccessorSegment(BaseSegment):
 
     type = "semi_structured_expression"
     match_grammar = Sequence(
-        Ref("DotSegment"),
-        Ref("SingleIdentifierGrammar"),
-        Ref("ArrayAccessorSegment", optional=True),
         AnyNumberOf(
             Sequence(
                 Ref("DotSegment"),
-                Ref("SingleIdentifierGrammar"),
+                OneOf(
+                    Ref("SingleIdentifierGrammar"),
+                    Ref("StarSegment"),
+                ),
                 allow_gaps=True,
             ),
             Ref("ArrayAccessorSegment", optional=True),
             allow_gaps=True,
+            min_times=1,
         ),
         allow_gaps=True,
     )
@@ -1213,7 +1243,7 @@ class TableReferenceSegment(ObjectReferenceSegment):
             AnyNumberOf(
                 Sequence(
                     Ref("DashSegment"),
-                    OneOf(Ref("SingleIdentifierGrammar"), Ref("NumericLiteralSegment")),
+                    Ref("NakedIdentifierPart"),
                     allow_gaps=False,
                 ),
                 optional=True,
@@ -1298,11 +1328,10 @@ class DeclareStatementSegment(BaseSegment):
         "DECLARE",
         Delimited(Ref("SingleIdentifierFullGrammar")),
         OneOf(
-            Ref("DatatypeSegment"),
             Ref("DefaultDeclareOptionsGrammar"),
             Sequence(
                 Ref("DatatypeSegment"),
-                Ref("DefaultDeclareOptionsGrammar"),
+                Ref("DefaultDeclareOptionsGrammar", optional=True),
             ),
         ),
     )
@@ -1349,12 +1378,7 @@ class PartitionBySegment(BaseSegment):
     """PARTITION BY partition_expression."""
 
     type = "partition_by_segment"
-    match_grammar = StartsWith(
-        "PARTITION",
-        terminator=OneOf("CLUSTER", "OPTIONS", "AS", Ref("DelimiterGrammar")),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "PARTITION",
         "BY",
         Ref("ExpressionSegment"),
@@ -1365,12 +1389,7 @@ class ClusterBySegment(BaseSegment):
     """CLUSTER BY clustering_column_list."""
 
     type = "cluster_by_segment"
-    match_grammar = StartsWith(
-        "CLUSTER",
-        terminator=OneOf("OPTIONS", "AS", Ref("DelimiterGrammar")),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "CLUSTER",
         "BY",
         Delimited(Ref("ExpressionSegment")),
@@ -1405,9 +1424,7 @@ class ColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
     match_grammar: Matchable = Sequence(
         Ref("SingleIdentifierGrammar"),  # Column name
         Ref("DatatypeSegment"),  # Column type
-        AnyNumberOf(
-            Ref("ColumnConstraintSegment", optional=True),
-        ),
+        AnyNumberOf(Ref("ColumnConstraintSegment")),
         Ref("OptionsSegment", optional=True),
     )
 
@@ -1424,7 +1441,7 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         Ref("IfNotExistsGrammar", optional=True),
         Ref("TableReferenceSegment"),
         Sequence(
-            OneOf("COPY", "LIKE"),
+            OneOf("COPY", "LIKE", "CLONE"),
             Ref("TableReferenceSegment"),
             optional=True,
         ),
@@ -1432,14 +1449,10 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         Sequence(
             Bracketed(
                 Delimited(
-                    OneOf(
-                        Ref("TableConstraintSegment"),
-                        Ref("ColumnDefinitionSegment"),
-                    ),
+                    Ref("ColumnDefinitionSegment"),
                     allow_trailing=True,
                 )
             ),
-            Ref("CommentClauseSegment", optional=True),
             optional=True,
         ),
         Ref("PartitionBySegment", optional=True),
@@ -1454,6 +1467,91 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
     )
 
 
+class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
+    """A `ALTER TABLE` statement."""
+
+    match_grammar = Sequence(
+        "ALTER",
+        "TABLE",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        OneOf(
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_set_options_statement
+            Sequence(
+                "SET",
+                Ref("OptionsSegment"),
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_add_column_statement
+            Delimited(
+                Sequence(
+                    "ADD",
+                    "COLUMN",
+                    Ref("IfNotExistsGrammar", optional=True),
+                    Ref("ColumnDefinitionSegment"),
+                ),
+                allow_trailing=True,
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_rename_to_statement
+            Sequence(
+                "RENAME",
+                "TO",
+                Ref("TableReferenceSegment"),
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_rename_column_statement
+            Delimited(
+                Sequence(
+                    "RENAME",
+                    "COLUMN",
+                    Ref("IfExistsGrammar", optional=True),
+                    Ref("SingleIdentifierGrammar"),  # Column name
+                    "TO",
+                    Ref("SingleIdentifierGrammar"),  # Column name
+                ),
+                allow_trailing=True,
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_drop_column_statement
+            Delimited(
+                Sequence(
+                    "DROP",
+                    "COLUMN",
+                    Ref("IfExistsGrammar", optional=True),
+                    Ref("SingleIdentifierGrammar"),  # Column name
+                ),
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_column_set_options_statement
+            Delimited(
+                Sequence(
+                    "ALTER",
+                    "COLUMN",
+                    Ref("IfExistsGrammar", optional=True),
+                    Ref("SingleIdentifierGrammar"),  # Column name
+                    OneOf(
+                        Sequence(
+                            "SET",
+                            OneOf(
+                                Ref("OptionsSegment"),
+                                Sequence(
+                                    "DATA",
+                                    "TYPE",
+                                    Ref("DatatypeSegment"),
+                                ),
+                                Sequence(
+                                    "DEFAULT",
+                                    OneOf(
+                                        Ref("LiteralGrammar"),
+                                        Ref("FunctionSegment"),
+                                    ),
+                                ),
+                            ),
+                        ),
+                        Sequence("DROP", OneOf("DEFAULT", Sequence("NOT", "NULL"))),
+                    ),
+                ),
+            ),
+        ),
+    )
+
+
 class CreateExternalTableStatementSegment(BaseSegment):
     """A `CREATE EXTERNAL TABLE` statement.
 
@@ -1476,7 +1574,7 @@ class CreateExternalTableStatementSegment(BaseSegment):
             ),
             optional=True,
         ),
-        # Although not specified in the BigQuery documentation optinal arguments for
+        # Although not specified in the BigQuery documentation optional arguments for
         # CREATE EXTERNAL TABLE statements can be ordered arbitrarily.
         AnyNumberOf(
             # connection names have the same rules as table names in BigQuery
@@ -1519,6 +1617,24 @@ class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
     )
 
 
+class AlterViewStatementSegment(BaseSegment):
+    """A `ALTER VIEW` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_view_set_options_statement
+    """
+
+    type = "alter_view_statement"
+
+    match_grammar = Sequence(
+        "ALTER",
+        "VIEW",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        "SET",
+        Ref("OptionsSegment"),
+    )
+
+
 class CreateMaterializedViewStatementSegment(BaseSegment):
     """A `CREATE MATERIALIZED VIEW` statement.
 
@@ -1579,7 +1695,7 @@ class DropMaterializedViewStatementSegment(BaseSegment):
 
 
 class ParameterizedSegment(BaseSegment):
-    """BigQuery allows named and argument based parameters to help preven SQL Injection.
+    """BigQuery allows named and argument based parameters to prevent SQL Injection.
 
     https://cloud.google.com/bigquery/docs/parameterized-queries
     """
@@ -1588,6 +1704,17 @@ class ParameterizedSegment(BaseSegment):
     match_grammar = OneOf(Ref("AtSignLiteralSegment"), Ref("QuestionMarkSegment"))
 
 
+class PivotForClauseSegment(BaseSegment):
+    """The FOR part of a PIVOT expression.
+
+    Needed to avoid BaseExpressionElementGrammar swallowing up the IN part
+    """
+
+    type = "pivot_for_clause"
+    match_grammar = GreedyUntil("IN")
+    parse_grammar = Ref("BaseExpressionElementGrammar")
+
+
 class FromPivotExpressionSegment(BaseSegment):
     """A PIVOT expression.
 
@@ -1605,7 +1732,7 @@ class FromPivotExpressionSegment(BaseSegment):
                 ),
             ),
             "FOR",
-            Ref("SingleIdentifierGrammar"),
+            Ref("PivotForClauseSegment"),
             "IN",
             Bracketed(
                 Delimited(
@@ -1624,12 +1751,14 @@ class UnpivotAliasExpressionSegment(BaseSegment):
 
     type = "alias_expression"
     match_grammar = Sequence(
+        Indent,
         Ref.keyword("AS", optional=True),
         OneOf(
             Ref("SingleQuotedLiteralSegment"),
             Ref("DoubleQuotedLiteralSegment"),
             Ref("NumericLiteralSegment"),
         ),
+        Dedent,
     )
 
 
@@ -1648,16 +1777,22 @@ class FromUnpivotExpressionSegment(BaseSegment):
             optional=True,
         ),
         OneOf(
+            # single column unpivot
             Bracketed(
                 Ref("SingleIdentifierGrammar"),
                 "FOR",
                 Ref("SingleIdentifierGrammar"),
                 "IN",
                 Bracketed(
-                    Delimited(Ref("SingleIdentifierGrammar")),
-                    Ref("UnpivotAliasExpressionSegment", optional=True),
+                    Delimited(
+                        Sequence(
+                            Delimited(Ref("SingleIdentifierGrammar")),
+                            Ref("UnpivotAliasExpressionSegment", optional=True),
+                        ),
+                    ),
                 ),
             ),
+            # multi column unpivot
             Bracketed(
                 Bracketed(
                     Delimited(
@@ -1785,9 +1920,7 @@ class MergeInsertClauseSegment(ansi.MergeInsertClauseSegment):
             Indent,
             Ref("BracketedColumnReferenceListGrammar", optional=True),
             Dedent,
-            Indent,
             Ref("ValuesClauseSegment", optional=True),
-            Dedent,
         ),
         Sequence("INSERT", "ROW"),
     )
@@ -1805,7 +1938,7 @@ class DeleteStatementSegment(BaseSegment):
     match_grammar: Matchable = Sequence(
         "DELETE",
         Ref.keyword("FROM", optional=True),
-        Ref("ObjectReferenceSegment"),
+        Ref("TableReferenceSegment"),
         Ref("AliasExpressionSegment", optional=True),
         Ref("WhereClauseSegment", optional=True),
     )
@@ -1853,7 +1986,10 @@ class ExportStatementSegment(BaseSegment):
                         ),
                     ),
                     Ref("EqualsSegment"),
-                    Ref("QuotedLiteralSegment"),
+                    OneOf(
+                        Ref("SingleQuotedLiteralSegment"),
+                        Ref("DoubleQuotedLiteralSegment"),
+                    ),
                 ),
                 # Bool options
                 # Note: adding as own type, rather than keywords as convention with
