@@ -18,6 +18,7 @@ from sqlfluff.core.parser import (
     Matchable,
     TypedParser,
     OneOf,
+    OptionallyBracketed,
     Ref,
     RegexLexer,
     RegexParser,
@@ -26,6 +27,8 @@ from sqlfluff.core.parser import (
     StringLexer,
     StringParser,
     SymbolSegment,
+    Indent,
+    Dedent,
 )
 from sqlfluff.dialects.dialect_mysql_keywords import (
     mysql_reserved_keywords,
@@ -93,25 +96,45 @@ mysql_dialect.insert_lexer_matchers(
 # Set Keywords
 # Do not clear inherited unreserved ansi keywords. Too many are needed to parse well.
 # Just add MySQL unreserved keywords.
-mysql_dialect.sets("unreserved_keywords").update(
-    [n.strip().upper() for n in mysql_unreserved_keywords.split("\n")]
+mysql_dialect.update_keywords_set_from_multiline_string(
+    "unreserved_keywords", mysql_unreserved_keywords
 )
 
 mysql_dialect.sets("reserved_keywords").clear()
-mysql_dialect.sets("reserved_keywords").update(
-    [n.strip().upper() for n in mysql_reserved_keywords.split("\n")]
+mysql_dialect.update_keywords_set_from_multiline_string(
+    "reserved_keywords", mysql_reserved_keywords
 )
 
-# Remove these reserved keywords to avoid issue in interval.sql
-# TODO - resolve this properly
-mysql_dialect.sets("reserved_keywords").difference_update(
-    ["MINUTE_SECOND", "SECOND_MICROSECOND"]
+# Set the datetime units
+mysql_dialect.sets("datetime_units").clear()
+mysql_dialect.sets("datetime_units").update(
+    [
+        # https://github.com/mysql/mysql-server/blob/1bfe02bdad6604d54913c62614bde57a055c8332/sql/sql_yacc.yy#L12321-L12345
+        # interval:
+        "DAY_HOUR",
+        "DAY_MICROSECOND",
+        "DAY_MINUTE",
+        "DAY_SECOND",
+        "HOUR_MICROSECOND",
+        "HOUR_MINUTE",
+        "HOUR_SECOND",
+        "MINUTE_MICROSECOND",
+        "MINUTE_SECOND",
+        "SECOND_MICROSECOND",
+        "YEAR_MONTH",
+        # interval_time_stamp
+        "DAY",
+        "WEEK",
+        "HOUR",
+        "MINUTE",
+        "MONTH",
+        "QUARTER",
+        "SECOND",
+        "MICROSECOND",
+        "YEAR",
+    ]
 )
 
-# Remove this reserved keyword to avoid issue in create_table_primary_foreign_keys.sql
-# TODO - resolve this properly
-mysql_dialect.sets("reserved_keywords").difference_update(["INDEX"])
-
 
 mysql_dialect.replace(
     QuotedIdentifierSegment=TypedParser(
@@ -161,8 +184,6 @@ mysql_dialect.replace(
             "DATE",
             "TIME",
             "TIMESTAMP",
-            "DATETIME",
-            "INTERVAL",
             optional=True,
         ),
         OneOf(
@@ -262,8 +283,13 @@ class AliasExpressionSegment(BaseSegment):
 
     type = "alias_expression"
     match_grammar = Sequence(
+        Indent,
         Ref.keyword("AS", optional=True),
-        Ref("SingleIdentifierGrammar"),
+        OneOf(
+            Ref("SingleIdentifierGrammar"),
+            Ref("QuotedLiteralSegment"),
+        ),
+        Dedent,
     )
 
 
@@ -286,16 +312,20 @@ class ColumnDefinitionSegment(BaseSegment):
                     optional=True,
                 ),
                 Sequence(Sequence("NOT", optional=True), "NULL", optional=True),
-                Sequence("DEFAULT", optional=True),
-                OneOf(
-                    Sequence(
-                        OneOf("CURRENT_TIMESTAMP", "NOW"),
-                        Bracketed(
-                            Ref("NumericLiteralSegment", optional=True), optional=True
+                Sequence(
+                    "DEFAULT",
+                    OneOf(
+                        Sequence(
+                            OneOf("CURRENT_TIMESTAMP", "NOW"),
+                            Bracketed(
+                                Ref("NumericLiteralSegment", optional=True),
+                                optional=True,
+                            ),
                         ),
+                        Ref("NumericLiteralSegment"),
+                        Ref("QuotedLiteralSegment"),
+                        optional=True,
                     ),
-                    Ref("NumericLiteralSegment"),
-                    Ref("QuotedLiteralSegment"),
                     optional=True,
                 ),
                 Sequence(
@@ -578,12 +608,7 @@ class DeleteUsingClauseSegment(BaseSegment):
     """A `USING` clause froma `DELETE` Statement`."""
 
     type = "using_clause"
-    match_grammar = StartsWith(
-        "USING",
-        terminator=Ref("FromClauseTerminatorGrammar"),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "USING",
         Delimited(
             Ref("FromExpressionSegment"),
@@ -628,6 +653,16 @@ class DeleteStatementSegment(BaseSegment):
     )
 
 
+class ColumnConstraintSegment(ansi.ColumnConstraintSegment):
+    """A column option; each CREATE TABLE column can have 0 or more."""
+
+    match_grammar: Matchable = OneOf(
+        ansi.ColumnConstraintSegment.match_grammar,
+        Sequence("CHARACTER", "SET", Ref("NakedIdentifierSegment")),
+        Sequence("COLLATE", Ref("CollationReferenceSegment")),
+    )
+
+
 class IndexTypeGrammar(BaseSegment):
     """index_type in table_constraint."""
 
@@ -682,7 +717,9 @@ class TableConstraintSegment(BaseSegment):
     match_grammar = OneOf(
         Sequence(
             Sequence(  # [ CONSTRAINT <Constraint name> ]
-                "CONSTRAINT", Ref("ObjectReferenceSegment"), optional=True
+                "CONSTRAINT",
+                Ref("ObjectReferenceSegment", optional=True),
+                optional=True,
             ),
             OneOf(
                 # UNIQUE [INDEX | KEY] [index_name] [index_type] (key_part,...)
@@ -692,7 +729,7 @@ class TableConstraintSegment(BaseSegment):
                     OneOf("INDEX", "KEY", optional=True),
                     Ref("IndexReferenceSegment", optional=True),
                     Ref("IndexTypeGrammar", optional=True),
-                    Ref("BracketedColumnReferenceListGrammar"),
+                    Ref("BracketedKeyPartListGrammar"),
                     Ref("IndexOptionsSegment", optional=True),
                 ),
                 # PRIMARY KEY [index_type] (key_part,...) [index_option] ...
@@ -700,7 +737,7 @@ class TableConstraintSegment(BaseSegment):
                     Ref("PrimaryKeyGrammar"),
                     Ref("IndexTypeGrammar", optional=True),
                     # Columns making up PRIMARY KEY constraint
-                    Ref("BracketedColumnReferenceListGrammar"),
+                    Ref("BracketedKeyPartListGrammar"),
                     Ref("IndexOptionsSegment", optional=True),
                 ),
                 # FOREIGN KEY [index_name] (col_name,...) reference_definition
@@ -748,7 +785,7 @@ class TableConstraintSegment(BaseSegment):
             OneOf("INDEX", "KEY"),
             Ref("IndexReferenceSegment", optional=True),
             Ref("IndexTypeGrammar", optional=True),
-            Ref("BracketedColumnReferenceListGrammar"),
+            Ref("BracketedKeyPartListGrammar"),
             Ref("IndexOptionsSegment", optional=True),
         ),
         # {FULLTEXT | SPATIAL} [INDEX | KEY] [index_name] (key_part,...)
@@ -757,12 +794,43 @@ class TableConstraintSegment(BaseSegment):
             OneOf("FULLTEXT", "SPATIAL"),
             OneOf("INDEX", "KEY", optional=True),
             Ref("IndexReferenceSegment", optional=True),
-            Ref("BracketedColumnReferenceListGrammar"),
+            Ref("BracketedKeyPartListGrammar"),
             Ref("IndexOptionsSegment", optional=True),
         ),
     )
 
 
+class CreateIndexStatementSegment(ansi.CreateIndexStatementSegment):
+    """A `CREATE INDEX` statement.
+
+    https://dev.mysql.com/doc/refman/8.0/en/create-index.html
+    """
+
+    match_grammar = Sequence(
+        "CREATE",
+        OneOf("UNIQUE", "FULLTEXT", "SPATIAL", optional=True),
+        "INDEX",
+        Ref("IndexReferenceSegment"),
+        Ref("IndexTypeGrammar", optional=True),
+        "ON",
+        Ref("TableReferenceSegment"),
+        Ref("BracketedKeyPartListGrammar"),
+        Ref("IndexOptionsSegment", optional=True),
+        AnySetOf(
+            Sequence(
+                "ALGORITHM",
+                Ref("EqualsSegment", optional=True),
+                OneOf("DEFAULT", "INPLACE", "COPY"),
+            ),
+            Sequence(
+                "LOCK",
+                Ref("EqualsSegment", optional=True),
+                OneOf("DEFAULT", "NONE", "SHARED", "EXCLUSIVE"),
+            ),
+        ),
+    )
+
+
 class IntervalExpressionSegment(BaseSegment):
     """An interval expression segment.
 
@@ -772,15 +840,8 @@ class IntervalExpressionSegment(BaseSegment):
     type = "interval_expression"
     match_grammar = Sequence(
         "INTERVAL",
-        OneOf(
-            # The Numeric Version
-            Sequence(
-                Ref("ExpressionSegment"),
-                OneOf(Ref("QuotedLiteralSegment"), Ref("DatetimeUnitSegment")),
-            ),
-            # The String version
-            Ref("QuotedLiteralSegment"),
-        ),
+        Ref("ExpressionSegment"),
+        Ref("DatetimeUnitSegment"),
     )
 
 
@@ -828,13 +889,33 @@ mysql_dialect.add(
         "->>", SymbolSegment, type="column_path_operator"
     ),
     BooleanDynamicSystemVariablesGrammar=OneOf(
-        # Boolean dynamic system varaiables can be set to ON/OFF, TRUE/FALSE, or 0/1:
+        # Boolean dynamic system variables can be set to ON/OFF, TRUE/FALSE, or 0/1:
         # https://dev.mysql.com/doc/refman/8.0/en/dynamic-system-variables.html
         # This allows us to match ON/OFF & TRUE/FALSE as keywords and therefore apply
         # the correct capitalisation policy.
         OneOf("ON", "OFF"),
         OneOf("TRUE", "FALSE"),
     ),
+    # (key_part, ...)
+    # key_part: {col_name [(length)] | (expr)} [ASC | DESC]
+    # https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    # https://dev.mysql.com/doc/refman/8.0/en/alter-table.html
+    # https://dev.mysql.com/doc/refman/8.0/en/create-index.html
+    BracketedKeyPartListGrammar=Bracketed(
+        Delimited(
+            Sequence(
+                OneOf(
+                    Ref("ColumnReferenceSegment"),
+                    Sequence(
+                        Ref("ColumnReferenceSegment"),
+                        Bracketed(Ref("NumericLiteralSegment")),
+                    ),
+                    Bracketed(Ref("ExpressionSegment")),
+                ),
+                OneOf("ASC", "DESC", optional=True),
+            ),
+        ),
+    ),
 )
 
 
@@ -891,25 +972,28 @@ class RoleReferenceSegment(ansi.RoleReferenceSegment):
     https://dev.mysql.com/doc/refman/8.0/en/role-names.html
     """
 
-    match_grammar: Matchable = Sequence(
-        OneOf(
-            Ref("NakedIdentifierSegment"),
-            Ref("QuotedIdentifierSegment"),
-            Ref("SingleQuotedIdentifierSegment"),
-            Ref("DoubleQuotedLiteralSegment"),
-        ),
+    match_grammar: Matchable = OneOf(
         Sequence(
-            Ref("AtSignLiteralSegment"),
             OneOf(
                 Ref("NakedIdentifierSegment"),
                 Ref("QuotedIdentifierSegment"),
                 Ref("SingleQuotedIdentifierSegment"),
                 Ref("DoubleQuotedLiteralSegment"),
             ),
-            optional=True,
-            allow_gaps=False,
+            Sequence(
+                Ref("AtSignLiteralSegment"),
+                OneOf(
+                    Ref("NakedIdentifierSegment"),
+                    Ref("QuotedIdentifierSegment"),
+                    Ref("SingleQuotedIdentifierSegment"),
+                    Ref("DoubleQuotedLiteralSegment"),
+                ),
+                optional=True,
+                allow_gaps=False,
+            ),
+            allow_gaps=True,
         ),
-        allow_gaps=True,
+        "CURRENT_USER",
     )
 
 
@@ -1018,6 +1102,13 @@ class StatementSegment(ansi.StatementSegment):
             Ref("InsertRowAliasSegment"),
             Ref("FlushStatementSegment"),
             Ref("LoadDataSegment"),
+            Ref("ReplaceSegment"),
+            Ref("AlterDatabaseStatementSegment"),
+            Ref("ReturnStatementSegment"),
+        ],
+        remove=[
+            # handle CREATE SCHEMA in CreateDatabaseStatementSegment
+            Ref("CreateSchemaStatementSegment"),
         ],
     )
 
@@ -1264,7 +1355,12 @@ class AlterViewStatementSegment(BaseSegment):
         Ref("TableReferenceSegment"),
         Ref("BracketedColumnReferenceListGrammar", optional=True),
         "AS",
-        Ref("SelectStatementSegment"),
+        OptionallyBracketed(
+            OneOf(
+                Ref("SelectStatementSegment"),
+                Ref("SetExpressionSegment"),
+            )
+        ),
         Ref("WithCheckOptionSegment", optional=True),
     )
 
@@ -1292,7 +1388,12 @@ class CreateViewStatementSegment(BaseSegment):
         Ref("TableReferenceSegment"),
         Ref("BracketedColumnReferenceListGrammar", optional=True),
         "AS",
-        Ref("SelectStatementSegment"),
+        OptionallyBracketed(
+            OneOf(
+                Ref("SelectStatementSegment"),
+                Ref("SetExpressionSegment"),
+            )
+        ),
         Ref("WithCheckOptionSegment", optional=True),
     )
 
@@ -1412,7 +1513,7 @@ class IfExpressionStatement(BaseSegment):
 
 
 class DefinerSegment(BaseSegment):
-    """This is the body of a `CREATE FUNCTION` statement."""
+    """This is the body of a `CREATE FUNCTION` and `CREATE TRIGGER` statements."""
 
     type = "definer_segment"
 
@@ -2077,7 +2178,7 @@ class PurgeBinaryLogsStatementSegment(BaseSegment):
             Sequence(
                 "BEFORE",
                 OneOf(
-                    Ref("DateTimeLiteralGrammar"),
+                    Ref("ExpressionSegment"),
                 ),
             ),
         ),
@@ -2331,7 +2432,7 @@ class LoadDataSegment(BaseSegment):
         "INTO",
         "TABLE",
         Ref("TableReferenceSegment"),
-        Sequence("PARTITION", Ref("SelectPartitionClauseSegment"), optional=True),
+        Ref("SelectPartitionClauseSegment", optional=True),
         Sequence("CHARACTER", "SET", Ref("NakedIdentifierSegment"), optional=True),
         Sequence(
             OneOf("FIELDS", "COLUMNS"),
@@ -2370,6 +2471,68 @@ class LoadDataSegment(BaseSegment):
     )
 
 
+class ReplaceSegment(BaseSegment):
+    """A `REPLACE` statement.
+
+    As per https://dev.mysql.com/doc/refman/8.0/en/replace.html
+    """
+
+    type = "replace_statement"
+
+    match_grammar = Sequence(
+        "REPLACE",
+        OneOf("LOW_PRIORITY", "DELAYED", optional=True),
+        Sequence("INTO", optional=True),
+        Ref("TableReferenceSegment"),
+        Ref("SelectPartitionClauseSegment", optional=True),
+        OneOf(
+            Sequence(
+                Ref("BracketedColumnReferenceListGrammar", optional=True),
+                Ref("ValuesClauseSegment"),
+            ),
+            Ref("SetClauseListSegment"),
+            Sequence(
+                Ref("BracketedColumnReferenceListGrammar", optional=True),
+                OneOf(
+                    Ref("SelectableGrammar"),
+                    Sequence(
+                        "TABLE",
+                        Ref("TableReferenceSegment"),
+                    ),
+                ),
+            ),
+        ),
+    )
+
+
+class CreateTriggerStatementSegment(ansi.CreateTriggerStatementSegment):
+    """Create Trigger Statement.
+
+    As Specified in https://dev.mysql.com/doc/refman/8.0/en/create-trigger.html
+    """
+
+    # "DEFINED = user", optional
+    match_grammar = Sequence(
+        "CREATE",
+        Ref("DefinerSegment", optional=True),
+        "TRIGGER",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TriggerReferenceSegment"),
+        OneOf("BEFORE", "AFTER"),
+        OneOf("INSERT", "UPDATE", "DELETE"),
+        "ON",
+        Ref("TableReferenceSegment"),
+        Sequence("FOR", "EACH", "ROW"),
+        Sequence(
+            OneOf("FOLLOWS", "PRECEDES"), Ref("SingleIdentifierGrammar"), optional=True
+        ),
+        OneOf(
+            Ref("StatementSegment"),
+            Sequence("BEGIN", Ref("StatementSegment"), "END"),
+        ),
+    )
+
+
 class DropTriggerStatementSegment(ansi.DropTriggerStatementSegment):
     """A `DROP TRIGGER` Statement.
 
@@ -2406,3 +2569,114 @@ class ColumnReferenceSegment(ansi.ColumnReferenceSegment):
             ),
         ]
     )
+
+
+class CreateDatabaseStatementSegment(ansi.CreateDatabaseStatementSegment):
+    """A `CREATE DATABASE` statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/create-database.html
+    """
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        OneOf("DATABASE", "SCHEMA"),
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("DatabaseReferenceSegment"),
+        AnyNumberOf(Ref("CreateOptionSegment")),
+    )
+
+
+class CreateOptionSegment(BaseSegment):
+    """A database characteristic.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/create-database.html
+    """
+
+    type = "create_option_segment"
+    match_grammar = Sequence(
+        Ref.keyword("DEFAULT", optional=True),
+        OneOf(
+            Sequence(
+                "CHARACTER",
+                "SET",
+                Ref("EqualsSegment", optional=True),
+                Ref("NakedIdentifierSegment"),
+            ),
+            Sequence(
+                "COLLATE",
+                Ref("EqualsSegment", optional=True),
+                Ref("CollationReferenceSegment"),
+            ),
+            Sequence(
+                "ENCRYPTION",
+                Ref("EqualsSegment", optional=True),
+                Ref("QuotedLiteralSegment"),
+            ),
+        ),
+    )
+
+
+class AlterDatabaseStatementSegment(BaseSegment):
+    """A `ALTER DATABASE` statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/alter-database.html
+    """
+
+    type = "alter_database_statement"
+    match_grammar: Matchable = Sequence(
+        "ALTER",
+        OneOf("DATABASE", "SCHEMA"),
+        Ref("DatabaseReferenceSegment", optional=True),
+        AnyNumberOf(Ref("AlterOptionSegment")),
+    )
+
+
+class AlterOptionSegment(BaseSegment):
+    """A database characteristic.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/alter-database.html
+    """
+
+    type = "alter_option_segment"
+    match_grammar = Sequence(
+        OneOf(
+            Sequence(
+                Ref.keyword("DEFAULT", optional=True),
+                "CHARACTER",
+                "SET",
+                Ref("EqualsSegment", optional=True),
+                Ref("NakedIdentifierSegment"),
+            ),
+            Sequence(
+                Ref.keyword("DEFAULT", optional=True),
+                "COLLATE",
+                Ref("EqualsSegment", optional=True),
+                Ref("CollationReferenceSegment"),
+            ),
+            Sequence(
+                Ref.keyword("DEFAULT", optional=True),
+                "ENCRYPTION",
+                Ref("EqualsSegment", optional=True),
+                Ref("QuotedLiteralSegment"),
+            ),
+            Sequence(
+                "READ",
+                "ONLY",
+                Ref("EqualsSegment", optional=True),
+                OneOf("DEFAULT", Ref("NumericLiteralSegment")),
+            ),
+        ),
+    )
+
+
+class ReturnStatementSegment(BaseSegment):
+    """A RETURN statement.
+
+    As specified in https://dev.mysql.com/doc/refman/8.0/en/return.html
+    """
+
+    type = "return_statement"
+    match_grammar = Sequence(
+        "RETURN",
+        Ref("ExpressionSegment"),
+    )
