@@ -33,6 +33,18 @@ oracle_dialect.sets("reserved_keywords").update(
     ["COMMENT", "ON", "UPDATE", "INDEXTYPE", "PROMPT"]
 )
 
+oracle_dialect.sets("bare_functions").clear()
+oracle_dialect.sets("bare_functions").update(
+    [
+        "current_date",
+        "current_timestamp",
+        "dbtimezone",
+        "localtimestamp",
+        "sessiontimestamp",
+        "sysdate",
+        "systimestamp",
+    ]
+)
 
 oracle_dialect.insert_lexer_matchers(
     [
@@ -50,6 +62,19 @@ oracle_dialect.add(
     AtSignSegment=StringParser("@", SymbolSegment, type="at_sign"),
 )
 
+oracle_dialect.replace(
+    # https://docs.oracle.com/en/database/oracle/oracle-database/19/sqlrf/DROP-TABLE.html
+    DropBehaviorGrammar=Sequence(
+        Sequence(
+            "CASCADE",
+            "CONSTRAINTS",
+            optional=True,
+        ),
+        Ref.keyword("PURGE", optional=True),
+        optional=True,
+    ),
+)
+
 
 class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
     """An `ALTER TABLE` statement.
