@@ -97,7 +97,7 @@ postgres_docs_keywords = [
     ("BEGIN", "non-reserved"),
     ("BEGIN_FRAME", "not-keyword"),
     ("BEGIN_PARTITION", "not-keyword"),
-    ("BERNOULLI", "not-keyword"),
+    ("BERNOULLI", "non-reserved"),
     ("BETWEEN", "non-reserved-(cannot-be-function-or-type)"),
     ("BIGINT", "non-reserved-(cannot-be-function-or-type)"),
     ("BIGSERIAL", "non-reserved-(cannot-be-function-or-type)"),
@@ -926,6 +926,8 @@ postgres_nondocs_keywords = [
     ("DEPTH", "non-reserved"),
     ("DESCRIBE", "non-reserved"),
     ("DETERMINISTIC", "non-reserved"),
+    ("DISABLE_PAGE_SKIPPING", "non-reserved"),
+    ("EXECUTION", "not-keyword"),
     ("EXTENDED", "non-reserved"),
     ("FILE", "non-reserved"),
     ("FORCE_NOT_NULL", "non-reserved"),
@@ -935,6 +937,7 @@ postgres_nondocs_keywords = [
     ("HASH", "non-reserved"),
     ("ICU", "non-reserved"),
     ("IGNORE", "non-reserved"),
+    ("INDEX_CLEANUP", "non-reserved"),
     ("IS_TEMPLATE", "non-reserved"),
     ("JSON", "non-reserved"),
     ("LC_COLLATE", "non-reserved"),
@@ -953,6 +956,7 @@ postgres_nondocs_keywords = [
     ("NOREPLICATION", "non-reserved"),
     ("NOSUPERUSER", "non-reserved"),
     ("PLAIN", "non-reserved"),
+    ("PROCESS_TOAST", "non-reserved"),
     ("PROVIDER", "non-reserved"),
     ("PUBLIC", "non-reserved"),
     ("REMAINDER", "non-reserved"),
@@ -963,12 +967,12 @@ postgres_nondocs_keywords = [
     ("SETTINGS", "non-reserved"),
     ("SKIP_LOCKED", "non-reserved"),
     ("SUMMARY", "non-reserved"),
-    ("SUPERUSER", "non-reserverd"),
+    ("SUPERUSER", "non-reserved"),
     ("TIMETZ", "non-reserved"),
     ("TIMESTAMPTZ", "non-reserved"),
     ("TIMING", "non-reserved"),
     ("UNSAFE", "non-reserved"),
-    ("USAGE", "reserved"),
+    ("USAGE", "non-reserved"),
     ("WAL", "non-reserved"),
 ]
 
