@@ -24,6 +24,7 @@ from sqlfluff.core.parser import (
     RegexLexer,
     Sequence,
     StartsWith,
+    StringParser,
 )
 
 from sqlfluff.core.dialects import load_raw_dialect
@@ -35,7 +36,12 @@ teradata_dialect = ansi_dialect.copy_as("teradata")
 teradata_dialect.patch_lexer_matchers(
     [
         # so it also matches 1.
-        RegexLexer("numeric_literal", r"([0-9]+(\.[0-9]*)?)", CodeSegment),
+        RegexLexer(
+            "numeric_literal",
+            r"([0-9]+(\.[0-9]*)?)",
+            CodeSegment,
+            segment_kwargs={"type": "numeric_literal"},
+        ),
     ]
 )
 
@@ -72,15 +78,19 @@ teradata_dialect.sets("unreserved_keywords").update(
         "MAXVALUELENGTH",
         "MEETS",
         "MERGEBLOCKRATIO",
+        "NONE",
         "PERCENT",
         "PROFILE",
         "PROTECTION",
+        "QUERY_BAND",
         "QUIT",
         "RUN",
         "SAMPLE",
         "SEL",
         "SS",
         "STAT",
+        "STATS",
+        "STATISTICS",
         "SUMMARY",
         "THRESHOLD",
         "UC",
@@ -92,6 +102,41 @@ teradata_dialect.sets("reserved_keywords").update(["UNION", "TIMESTAMP"])
 
 teradata_dialect.sets("bare_functions").update(["DATE"])
 
+teradata_dialect.replace(
+    # ANSI standard comparison operators plus Teradata extensions
+    ComparisonOperatorGrammar=OneOf(
+        Ref("EqualsSegment"),
+        Ref("EqualsSegment_a"),
+        Ref("GreaterThanSegment"),
+        Ref("GreaterThanSegment_a"),
+        Ref("LessThanSegment"),
+        Ref("LessThanSegment_a"),
+        Ref("GreaterThanOrEqualToSegment"),
+        Ref("GreaterThanOrEqualToSegment_a"),
+        Ref("LessThanOrEqualToSegment"),
+        Ref("LessThanOrEqualToSegment_a"),
+        Ref("NotEqualToSegment"),
+        Ref("NotEqualToSegment_a"),
+        Ref("NotEqualToSegment_b"),
+        Ref("NotEqualToSegment_c"),
+        Ref("LikeOperatorSegment"),
+        Sequence("IS", "DISTINCT", "FROM"),
+        Sequence("IS", "NOT", "DISTINCT", "FROM"),
+    )
+)
+
+teradata_dialect.add(
+    # Add Teradata comparison operator extensions
+    EqualsSegment_a=StringParser("EQ", ansi.ComparisonOperatorSegment),
+    GreaterThanSegment_a=StringParser("GT", ansi.ComparisonOperatorSegment),
+    LessThanSegment_a=StringParser("LT", ansi.ComparisonOperatorSegment),
+    GreaterThanOrEqualToSegment_a=StringParser("GE", ansi.ComparisonOperatorSegment),
+    LessThanOrEqualToSegment_a=StringParser("LE", ansi.ComparisonOperatorSegment),
+    NotEqualToSegment_a=StringParser("NE", ansi.ComparisonOperatorSegment),
+    NotEqualToSegment_b=StringParser("NOT=", ansi.ComparisonOperatorSegment),
+    NotEqualToSegment_c=StringParser("^=", ansi.ComparisonOperatorSegment),
+)
+
 
 # BTEQ statement
 class BteqKeyWordSegment(BaseSegment):
@@ -208,7 +253,7 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
     match_grammar = Sequence(
         "COLLECT",
         Ref.keyword("SUMMARY", optional=True),
-        OneOf("STATISTICS", "STAT"),
+        OneOf("STAT", "STATS", "STATISTICS"),
         Sequence(
             "USING",
             Delimited(
@@ -238,7 +283,7 @@ class TdCollectStatisticsStatementSegment(BaseSegment):
                 # COLUMN ...
                 Sequence(
                     "COLUMN",
-                    Bracketed(
+                    OptionallyBracketed(
                         Delimited(
                             OneOf(
                                 Ref("ColumnReferenceSegment"),
@@ -335,6 +380,7 @@ class DatatypeSegment(ansi.DatatypeSegment):
 
     match_grammar = Sequence(
         Ref("DatatypeIdentifierSegment"),
+        Ref("BracketedArguments", optional=True),
         Bracketed(
             OneOf(
                 Delimited(Ref("ExpressionSegment")),
@@ -532,43 +578,52 @@ class TdTableConstraints(BaseSegment):
     """
 
     type = "td_table_constraint"
-    match_grammar = Sequence(
-        AnyNumberOf(
-            # PRIMARY Index
-            OneOf(
-                Sequence(  # UNIQUE PRIMARY INDEX Column_name | ( Column_name, ... )
-                    Ref.keyword("UNIQUE", optional=True),
-                    "PRIMARY",
-                    "INDEX",
-                    Ref("ObjectReferenceSegment", optional=True),  # primary index name
-                    OneOf(
-                        Bracketed(
-                            Delimited(
-                                Ref("SingleIdentifierGrammar"),
-                            )
-                        ),
-                        Ref("SingleIdentifierGrammar"),
-                    ),
-                ),
-                Sequence("NO", "PRIMARY", "INDEX"),  # NO PRIMARY INDEX
-            ),
-            # PARTITION BY ...
-            Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
-                "PARTITION",
-                "BY",
-                Ref("TdTablePartitioningLevel"),
-            ),
-            # Index
-            Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+    match_grammar = AnyNumberOf(
+        # PRIMARY Index
+        OneOf(
+            Sequence(  # UNIQUE PRIMARY INDEX Column_name | ( Column_name, ... )
                 Ref.keyword("UNIQUE", optional=True),
+                "PRIMARY",
                 "INDEX",
-                Ref("ObjectReferenceSegment"),  # Index name
-                Ref.keyword("ALL", optional=True),
-                Bracketed(  # Columns making up  constraint
-                    Delimited(Ref("ColumnReferenceSegment")),
+                Ref("ObjectReferenceSegment", optional=True),  # primary index name
+                OneOf(
+                    Bracketed(
+                        Delimited(
+                            Ref("SingleIdentifierGrammar"),
+                        )
+                    ),
+                    Ref("SingleIdentifierGrammar"),
                 ),
             ),
-        )
+            Sequence("NO", "PRIMARY", "INDEX"),  # NO PRIMARY INDEX
+        ),
+        # PARTITION BY ...
+        Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+            "PARTITION",
+            "BY",
+            Ref("TdTablePartitioningLevel"),
+        ),
+        # Index
+        Sequence(  # INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );
+            Ref.keyword("UNIQUE", optional=True),
+            "INDEX",
+            Ref("ObjectReferenceSegment"),  # Index name
+            Ref.keyword("ALL", optional=True),
+            Bracketed(  # Columns making up  constraint
+                Delimited(Ref("ColumnReferenceSegment")),
+            ),
+        ),
+        # WITH DATA
+        Sequence("WITH", Sequence("NO", optional=True), "DATA"),
+        # AND STATISITCS
+        Sequence(
+            "AND",
+            Sequence("NO", optional=True),
+            OneOf("STAT", "STATS", "STATISTICS"),
+            optional=True,
+        ),
+        # ON COMMIT PRESERVE ROWS
+        Sequence("ON", "COMMIT", OneOf("PRESERVE", "DELETE"), "ROWS"),
     )
 
 
@@ -641,8 +696,7 @@ class FromUpdateClauseSegment(BaseSegment):
     """A `FROM` clause like in `SELECT` but terminated by SET."""
 
     type = "from_in_update_clause"
-    match_grammar = StartsWith("FROM", terminator=Ref.keyword("SET"))
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "FROM",
         Delimited(
             # Optional old school delimited joins
@@ -666,24 +720,14 @@ class StatementSegment(ansi.StatementSegment):
             Ref("TdCommentStatementSegment"),
             Ref("DatabaseStatementSegment"),
             Ref("SetSessionStatementSegment"),
+            Ref("SetQueryBandStatementSegment"),
         ],
     )
 
     match_grammar = ansi.StatementSegment.match_grammar.copy()
 
 
-teradata_dialect.add(
-    TdCastIdentifierGrammar=Sequence(
-        OneOf("DATE", "TIMESTAMP"), Ref("ExpressionSegment")
-    ),
-)
-
 teradata_dialect.replace(
-    SingleIdentifierGrammar=OneOf(
-        Ref("NakedIdentifierSegment"),
-        Ref("QuotedIdentifierSegment"),
-        Ref("TdCastIdentifierGrammar"),
-    ),
     SelectClauseSegmentGrammar=Sequence(
         OneOf("SELECT", "SEL"),
         Ref("SelectClauseModifierSegment", optional=True),
@@ -700,12 +744,7 @@ class QualifyClauseSegment(BaseSegment):
     """A `QUALIFY` clause like in `SELECT`."""
 
     type = "qualify_clause"
-    match_grammar = StartsWith(
-        "QUALIFY",
-        terminator=OneOf(Sequence("ORDER", "BY"), "LIMIT", "QUALIFY", "WINDOW"),
-        enforce_whitespace_preceding_terminator=True,
-    )
-    parse_grammar = Sequence(
+    match_grammar = Sequence(
         "QUALIFY",
         Indent,
         OptionallyBracketed(Ref("ExpressionSegment")),
@@ -750,6 +789,12 @@ class SelectClauseModifierSegment(BaseSegment):
     match_grammar = OneOf(
         "DISTINCT",
         "ALL",
+        Sequence(
+            "TOP",
+            Ref("ExpressionSegment"),
+            Sequence("PERCENT", optional=True),
+            Sequence("WITH", "TIES", optional=True),
+        ),
         Sequence(
             "NORMALIZE",
             OneOf(
@@ -826,3 +871,46 @@ class SetSessionStatementSegment(BaseSegment):
         ),
         Ref("DatabaseStatementSegment"),
     )
+
+
+class SetQueryBandStatementSegment(BaseSegment):
+    """A `SET QUERY_BAND` statement.
+
+    SET QUERY_BAND = { 'band_specification [...]' | NONE } [ UPDATE ]
+    FOR { SESSION [VOLATILE] | TRANSACTION } [;]
+
+    https://docs.teradata.com/r/Teradata-VantageTM-SQL-Data-Definition-Language-Syntax-and-Examples/July-2021/Session-Statements/SET-QUERY_BAND
+    """
+
+    type = "set_query_band_statement"
+    match_grammar: Matchable = Sequence(
+        "SET",
+        "QUERY_BAND",
+        Ref("EqualsSegment"),
+        OneOf(Ref("QuotedLiteralSegment"), "NONE"),
+        Sequence("UPDATE", optional=True),
+        "FOR",
+        OneOf(Sequence("SESSION", Sequence("VOLATILE", optional=True)), "TRANSACTION"),
+    )
+
+
+class NotEqualToSegment_b(ansi.CompositeComparisonOperatorSegment):
+    """The comparison operator extension NOT=.
+
+    https://www.docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/Supported-Comparison-Operators
+    """
+
+    match_grammar = Sequence(
+        Ref("NotOperatorGrammar"), Ref("RawEqualsSegment"), allow_gaps=False
+    )
+
+
+class NotEqualToSegment_c(ansi.CompositeComparisonOperatorSegment):
+    """The comparison operator extension ^=.
+
+    https://www.docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/Supported-Comparison-Operators
+    """
+
+    match_grammar = Sequence(
+        Ref("BitwiseXorSegment"), Ref("RawEqualsSegment"), allow_gaps=False
+    )
