@@ -14,6 +14,7 @@ RESERVED_KEYWORDS = [
     "ASC",
     "AUTHORIZATION",
     "BACKUP",
+    "BATCHSIZE",
     "BEGIN",
     "BETWEEN",
     "BREAK",
@@ -24,6 +25,7 @@ RESERVED_KEYWORDS = [
     "CASE",
     "CHECK",
     "CHECKPOINT",
+    "CHECK_CONSTRAINTS",
     "CLOSE",
     "CLUSTERED",
     "COALESCE",
@@ -38,13 +40,14 @@ RESERVED_KEYWORDS = [
     "CONVERT",
     "CREATE",
     "CROSS",
+    "CURRENT",
     "CURRENT_DATE",
     "CURRENT_TIME",
     "CURRENT_TIMESTAMP",
     "CURRENT_USER",
-    "CURRENT",
     "CURSOR",
     "DATABASE",
+    "DAY",
     "DBCC",
     "DEALLOCATE",
     "DECLARE",
@@ -78,6 +81,7 @@ RESERVED_KEYWORDS = [
     "FREETEXTTABLE",
     "FROM",
     "FULL",
+    "FULLSCAN",
     "FUNCTION",
     "GLOBAL",
     "GO",
@@ -143,6 +147,7 @@ RESERVED_KEYWORDS = [
     "RECONFIGURE",
     "REFERENCES",
     "REPLICATION",
+    "RESAMPLE",
     "RESTORE",
     "RESTRICT",
     "RETURN",
@@ -204,6 +209,8 @@ RESERVED_KEYWORDS = [
 
 UNRESERVED_KEYWORDS = [
     "ABORT_AFTER_WAIT",
+    "ABORT",
+    "ABSENT",
     "AFTER",
     "ALGORITHM",
     "ALLOW_PAGE_LOCKS",
@@ -215,12 +222,14 @@ UNRESERVED_KEYWORDS = [
     "ANSI_NULLS",
     "ANSI_PADDING",
     "ANSI_WARNINGS",
+    "APPEND_ONLY",
     "APPLY",
     "ARITHABORT",
     "ARITHIGNORE",
     "AT",
     "AUTO",
     "BERNOULLI",
+    "BINARY",
     "BLOCKERS",
     "BREAK",
     "CACHE",
@@ -230,23 +239,35 @@ UNRESERVED_KEYWORDS = [
     "CATCH",
     "CODEPAGE",
     "COLUMN_ENCRYPTION_KEY",
-    "COLUMNSTORE",
     "COLUMNSTORE_ARCHIVE",
+    "COLUMNSTORE",
     "COMMITTED",
-    "CONCAT",
+    "COMPRESS_ALL_ROW_GROUPS",
+    "COMPRESSION_DELAY",
     "CONCAT_NULL_YIELDS_NULL",
+    "CONCAT",
+    "CONNECTION_OPTIONS",
+    "CONTAINED",
     "CONTINUE",
     "CONTROL",
-    "COMPRESSION_DELAY",
+    "CREDENTIAL",
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
     "DATA_COMPRESSION",
+    "DATA_CONSISTENCY_CHECK",
+    "DATA_DELETION",
+    "DATA_SOURCE",
+    "DATA",
+    "DATAFILETYPE",
     "DATASOURCE",
+    "DATE_FORMAT",
     "DATE",
     "DATEFIRST",
     "DATEFORMAT",
     "DEADLOCK_PRIORITY",
     "DELAY",
+    "DELIMITEDTEXT",
+    "DELTA",
     "DENSE_RANK",
     "DETERMINISTIC",
     "DISABLE",
@@ -254,21 +275,37 @@ UNRESERVED_KEYWORDS = [
     "DISTRIBUTION",  # Azure Synapse Analytics specific
     "DROP_EXISTING",
     "DUMP",  # listed as reserved but functionally unreserved
+    "DURABILITY",
+    "ELEMENTS",
+    "ENCODING",
     "ENCRYPTED",
-    "ENCRYPTION",
     "ENCRYPTION_TYPE",
-    "ERRORFILE",
+    "ENCRYPTION",
     "ERRORFILE_DATA_SOURCE",
+    "ERRORFILE",
     "EXPAND",
     "EXPLAIN",  # Azure Synapse Analytics specific
     "EXPLICIT",
     "EXTERNALPUSHDOWN",
     "FAST",
+    "FIELD_TERMINATOR",
     "FIELDQUOTE",
+    "FIELDTERMINATOR",
+    "FILE_FORMAT",
     "FILESTREAM",
     "FILESTREAM_ON",
+    "FILESTREAM",
+    "FILETABLE_COLLATE_FILENAME",
+    "FILETABLE_DIRECTORY",
+    "FILETABLE_FULLPATH_UNIQUE_CONSTRAINT_NAME",
+    "FILETABLE_PRIMARY_KEY_CONSTRAINT_NAME",
+    "FILETABLE_STREAMID_UNIQUE_CONSTRAINT_NAME",
+    "FILTER_COLUMN",
+    "FILTER_PREDICATE",
     "FILTER",
     "FIPS_FLAGGER",
+    "FIRE_TRIGGERS",
+    "FIRST_ROW",
     "FIRST",
     "FIRSTROW",
     "FMTONLY",
@@ -278,19 +315,26 @@ UNRESERVED_KEYWORDS = [
     "FORCEPLAN",
     "FORCESCAN",
     "FORCESEEK",
+    "FORMAT_OPTIONS",
+    "FORMAT_TYPE",
     "FORMAT",
-    "FORMATFILE",
     "FORMATFILE_DATA_SOURCE",
+    "FORMATFILE",
+    "GENERATED",
     "HASH",
     "HEAP",  # Azure Synapse Analytics specific
     "HIDDEN",
     "HINT",
-    "IGNORE",
+    "HISTORY_RETENTION_PERIOD",
+    "HISTORY_TABLE",
     "IGNORE_CONSTRAINTS",
     "IGNORE_DUP_KEY",
     "IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX",
     "IGNORE_TRIGGERS",
+    "IGNORE",
     "IMPLICIT_TRANSACTIONS",
+    "INBOUND",
+    "INCLUDE_NULL_VALUES",
     "INCLUDE",
     "INCREMENT",
     "INLINE",
@@ -298,39 +342,44 @@ UNRESERVED_KEYWORDS = [
     "INTERVAL",
     "IO",
     "ISOLATION",
-    "GENERATED",
+    "JSON",
     "KEEP",
     "KEEPDEFAULTS",
     "KEEPFIXED",
     "KEEPIDENTITY",
-    # LABEL is an Azure Synapse Analytics specific reserved keyword
-    # but could break TSQL parsing to add there
-    "LABEL",
+    "KEEPNULLS",
+    "KILOBYTES_PER_BATCH",
+    "LABEL",  # *reserved* keyword in Azure Synapse; but would break TSQL parsing
     "LANGUAGE",
     "LAST",
     "LASTROW",
+    "LEDGER_VIEW",
+    "LEDGER",
     "LEVEL",
     "LOAD",  # listed as reserved but functionally unreserved
+    "LOB_COMPACTION",
     "LOCATION",
     "LOCK_TIMEOUT",
     "LOG",
     "LOGIN",
     "LOOP",
-    "MAX_DURATION",
-    "MAX_GRANT_PERCENT",
     "MASKED",
     "MATCHED",
+    "MAX_DURATION",
+    "MAX_GRANT_PERCENT",
     "MAX",
     "MAXDOP",
     "MAXERRORS",
     "MAXRECURSION",
     "MAXVALUE",
+    "MEMORY_OPTIMIZED",
+    "MIGRATION_STATE",
     "MIN_GRANT_PERCENT",
     "MINUTES",
     "MINVALUE",
     "NEXT",
-    "NO",
     "NO_PERFORMANCE_SPOOL",
+    "NO",
     "NOCOUNT",
     "NOEXEC",
     "NOEXPAND",
@@ -342,9 +391,14 @@ UNRESERVED_KEYWORDS = [
     "OBJECT",
     "OFFSET",
     "ONLINE",
-    "OPTIMIZE",
+    "OPENJSON",
+    "OPERATION_TYPE_COLUMN_NAME",
+    "OPERATION_TYPE_DESC_COLUMN_NAME",
     "OPTIMIZE_FOR_SEQUENTIAL_KEY",
+    "OPTIMIZE",
+    "ORC",
     "OUT",
+    "OUTBOUND",
     "OUTPUT",
     "OWNER",
     "PAD_INDEX",
@@ -352,34 +406,50 @@ UNRESERVED_KEYWORDS = [
     "PAGLOCK",
     "PARAMETER",
     "PARAMETERIZATION",
+    "PARQUET",
     "PARSEONLY",
+    "PARSER_VERSION",
     "PARTITION",
     "PARTITIONS",
     "PATH",
+    "PAUSE",
+    "PAUSED",
+    "PERCENTAGE",
     "PERCENTILE_CONT",
     "PERCENTILE_DISC",
+    "PERIOD",
     "PERSISTED",
     "PRECEDING",
     "PRECISION",  # listed as reserved but functionally unreserved
     "PRIOR",
     "PROFILE",
+    "PUSHDOWN",
     "QUERY_GOVERNOR_COST_LIMIT",
     "QUERYTRACEON",
     "QUOTED_IDENTIFIER",
+    "R",  # sqlcmd command
     "RANDOMIZED",
     "RANGE",
     "RANK",
     "RAW",
+    "RCFILE",
     "READCOMMITTED",
     "READCOMMITTEDLOCK",
     "READONLY",
     "READPAST",
     "READUNCOMMITTED",
+    "REBUILD",
     "RECEIVE",
     "RECOMPILE",
     "RECURSIVE",
+    "REJECT_SAMPLE_VALUE",
+    "REJECT_TYPE",
+    "REJECT_VALUE",
+    "REJECTED_ROW_LOCATION",
+    "REMOTE_DATA_ARCHIVE",
     "REMOTE_PROC_TRANSACTIONS",
     "RENAME",  # Azure Synapse Analytics specific
+    "REORGANIZE",
     "REPEATABLE",
     "REPEATABLEREAD",
     "REPLACE",
@@ -387,25 +457,37 @@ UNRESERVED_KEYWORDS = [
     "RESPECT",
     "RESULT_SET_CACHING",  # Azure Synapse Analytics specific
     "RESUMABLE",
+    "RESUME",
+    "RETENTION_PERIOD",
     "RETURNS",
     "ROBUST",
     "ROLE",
+    "ROOT",
     "ROUND_ROBIN",  # Azure Synapse Analytics specific
-    "ROW",
     "ROW_NUMBER",
+    "ROW",
     "ROWGUIDCOL",
     "ROWLOCK",
+    "ROWS_PER_BATCH",
     "ROWS",
+    "ROWTERMINATOR",
     "S",
     "SCALEOUTEXECUTION",
+    "SCHEMA_AND_DATA",
+    "SCHEMA_ONLY",
     "SCHEMABINDING",
+    "SCOPED",
+    "SECRET",
     "SECURITYAUDIT",  # listed as reserved but functionally unreserved
     "SELF",
-    "SETERROR",
-    "SEQUENCE",
+    "SEQUENCE_NUMBER_COLUMN_NAME",
     "SEQUENCE_NUMBER",
+    "SEQUENCE",
+    "SERDE_METHOD",
     "SERIALIZABLE",
     "SERVER",
+    "SETERROR",
+    "SETVAR",  # sqlcmd command
     "SHOWPLAN_ALL",
     "SHOWPLAN_TEXT",
     "SHOWPLAN_XML",
@@ -421,7 +503,11 @@ UNRESERVED_KEYWORDS = [
     "STATISTICS_INCREMENTAL",
     "STATISTICS_NORECOMPUTE",
     "STRING_AGG",
+    "STRING_DELIMITER",
     "SWITCH",
+    "SYNONYM",
+    "SYSTEM_TIME",
+    "SYSTEM_VERSIONING",
     "SYSTEM",
     "TABLOCK",
     "TABLOCKX",
@@ -433,25 +519,32 @@ UNRESERVED_KEYWORDS = [
     "TIME",
     "TIMEOUT",
     "TIMESTAMP",
+    "TRANSACTION_ID_COLUMN_NAME",
     "TRANSACTION_ID",
     "TRUNCATE_TARGET",  # Azure Synapse Analytics specific
     "TRY",
     "TYPE",
-    "UPDLOCK",
     "UNBOUNDED",
     "UNCOMMITTED",
     "UNKNOWN",
+    "UPDLOCK",
+    "USE_TYPE_DEFAULT",
     "USER_DB",  # Azure Synapse Analytics specific, deprecated
     "USING",
     "VALUE",
     "VIEW_METADATA",
-    "WAITFOR",
     "WAIT_AT_LOW_PRIORITY",
-    "WITHIN",
+    "WAITFOR",
     "WHILE",
+    "WITHIN",
+    "WITHOUT_ARRAY_WRAPPER",
     "WORK",
     "XACT_ABORT",
     "XLOCK",
+    "XML_COMPRESSION",
     "XML",
+    "XMLDATA",
+    "XMLSCHEMA",
+    "XSINIL",
     "ZONE",
 ]
